#!/usr/bin/env bash
#set -x
############################################################################################### 
# Licensed Materials - Property of IBM
#
# 5724-T60 5608-A91
# (C) Copyright IBM Corp. 2008, 2009  All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication, or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###############################################################################################
# Verify input
#TODO:need detailed usage statement

printusage() {
echo "usage: removeDb2Inst DB2_HOME DB2_INST DB2_INST_HOME DB_NAME
  where
  DB2_HOME is the directory where DB2 is installed
  DB2_INST is the Database Instance Owner specified during the install
  DB2_INST_HOME is the home directory of the database instance owner
  DB2_NAME is the name of the database"
  exit 1
}

SCRIPTHOME=${PWD}
TKLMTEMP=${HOME}/sklmV25properties

if [ $# -le 0 ]; then
    if [[ -f ${SCRIPTHOME}/db2unix.srcit ]]; then
           . ${SCRIPTHOME}/"db2unix.srcit"
      else
           echo "The DB2 variables cannot be determined. The ${SCRIPTHOME}/db2unix.srcit is missing. Run cript again by specifying DB2 parameters."
	   printusage
	   exit 1 
     fi
else
  export DB2INSTALLDIR="$1"
   export DB2ADMIN="$2"
   export INSTANCEHOME="$3"
   export DB2DBNAME="$4"
fi

#echo "DB2 HOME "  $DB2INSTALLDIR 
#echo "DB2 INSTANCE NAME " $DB2ADMIN
#echo "DB2 INSTANCE HOME " $INSTANCEHOME
#echo "DB2 DATABASE NAME " $DB2DBNAME


if [ "$DB2INSTALLDIR" =  "" ]; then
   printusage
fi
if [ "$DB2ADMIN" =  "" ]; then
   printusage
fi

if [ "$INSTANCEHOME" =  "" ]; then
   printusage
fi

if [ "$DB2DBNAME" =  "" ]; then
   printusage
fi

#Warn the user about losing data
#ask=0
#while ((ask>0))
#do
#echo  "\nYou will lose the database instance and all data in it when you run this shell script, are you sure you want to continue?(yes/no) "
#read response 
#if [ "$response" = "no" ]; then 
#echo 
#echo "Uninstall is cancelled."
#echo
#exit 1
#fi
#if [ "$response" = "yes" ]; then 
#	ask=0
#else
#   echo 
#   echo "You specified ${response}. Enter yes to continue with the uninstallation and no to cancel uninstallation"
#   echo
#fi
#done

#echo "Removing database instance..."
#echo "DB2 Instance uninstall started" > /${TKLMTEMP}/removeDB2Inst.output.txt

if [ -d "${DB2INSTALLDIR}" ]; then
  . ${INSTANCEHOME}/sqllib/db2profile
  db2stop force >> /${TKLMTEMP}/removeDB2Inst.output.txt
  db2start >> /${TKLMTEMP}/removeDB2Inst.output.txt
  db2 drop database $DB2DBNAME >> /${TKLMTEMP}/removeDB2Inst.output.txt
  db2stop  >> /${TKLMTEMP}/removeDB2Inst.output.txt

  $DB2INSTALLDIR/instance/db2idrop  $DB2ADMIN >> /${TKLMTEMP}/removeDB2Inst.output.txt
  RC=$?
else
  echo "Error:" $DB2INSTALLDIR " does not exist"
  RC=-10
fi

if [ RC -eq 0 ]; then
  rm -rf $INSTANCEHOME/sqllib
  rm -rf $INSTANCEHOME/sqllib_v91
  rm -rf $INSTANCEHOME/$DB2ADMIN
  rm -rf $INSTANCEHOME/sklmdbarchive 
  RC=$?
fi

#if [ RC -eq 0 ]; then
#  echo "Uninstall DB2 instance is complete. You can look at /${TKLMTEMP}/removeDB2Inst.output.txt for more information"
#else
#  echo "Uninstall DB2 instance failed. You can look at /${TKLMTEMP}/removeDB2Inst.output.txt for more information"
#  echo "exiting with " $RC
#fi
#exit $RC
