#!/bin/sh
#set -x
#
# Licensed Materials-Property of IBM
# (C)Copyright IBM Corp 2008
# All rights reserved
# US Government Users Restricted Rights
# Use, duplication, or disclosure restricted
# by GSA ADP Schedule Contract with IBM Corp
# 
# The script is to be called from the IBM Tivoli Key Life Cycle Manager 
# Installer
#
# arg1 -  log  -   log file to be used by this script 
# arg2 -  instance  -  instance name (for example db2inst2) 
# arg3 -  dbname  -  database name
# arg4 -  dbuserid  - database instance owner
# arg5 -  dbpassword  - database password
# arg6 -  dbport - - database port 
# arg7 -  dblocation - path where Db2 is installed (for example /opt/IBM/db2/V9.1
# arg8 -  dbinstance_owner_home - (For example /home/db2inst2)
#
#  Note:  The script 
#	        must be run on the same machine where DB2 server is installed.
#		does do not parameter validation
#	  	will fail if the user specifies a wrong dbuserid or password
#	        will drop the database instance and database it created 
#	        will not drop database or instance created by ezinstall.
#	        uses the dbusername for the db2fence ID
#		uses the database name as it alias
#
#------------------------------------------------------------------------------

LOG=$1
DB_INST=$2
DB_NAME=$3
DB_USERID=$4
DB_PASSWORD=$5
DB_PORT=$6
DBPATH=$7
INST_HOME=$8
CREATEINSTDB=0
SVCE_NAME=db2c_${DB_INST}
dbarchdir=$INST_HOME/sklmdbarchive

#flag to indicate that we created the instance
inst_created=0
#flag to indicate that we created the Tivoli Key Lifecycle Manager database
db_created=0
#flag to indicate if we are connected to the database
is_connected=0
#dir created
dir_created=0
#The return code from the script
rc=0

if [ $# -ne 8 ]
then
echo Usage: dbtklmcr.sh logfile dbinstance dbname dbusername dbpassword dbport dblocation dbinstance_owner_home
    exit 2
fi


##################################################################
echo "---------------------------------------------------------------------------------------------------" >> $LOG
echo Started executing dbtklmcr.sh >> "$LOG"
echo "---------------------------------------------------------------------------------------------------" >> $LOG

CURRENT_DIR=`dirname $0`
if [ "$CURRENT_DIR" = "." ] ; then
   CURRENT_DIR=`pwd`
fi


# set instance profile path to source profile of local instance

#source $DB_INST/db2profile
echo . $INST_HOME/sqllib/db2profile >> "$LOG" 
. $INST_HOME/sqllib/db2profile
rc=$?
   

if [ -d $dbarchdir ]; then 
   echo "$dbarchdir directory exists. DB2 will use this directory for saving DB2 non active transactional logs." >> "$LOG"
else
   mkdir $dbarchdir
   rc=$?
   if [ $rc = 0 ]; then
     dir_created=1
     echo "Created $dbarchdir directory created. DB2 will use this directory for saving DB2 non active transactional logs." >> "$LOG"
     chown $DB_INST $dbarchdir 
     rc=$?
   fi
fi


if [ $rc -eq 0 ]; then
   echo db2 start database manager >> "$LOG"
   db2 -l "$LOG" "start database manager"
   #Just assume database manager started
fi

#Create the database
if [ "$CREATEINSTDB" = "0" ]; then
   echo db2 create database $DB_NAME using codeset UTF-8 territory US >> "$LOG"
   db2 -l "$LOG" "create database $DB_NAME using codeset UTF-8 territory US"
   rc=$?
   if [ $rc -eq 0 ] ; then 
      db_created=1
   fi
fi

#Connect to the database specifying the id and the password
if [ $rc -eq 0 ]; then 
    echo "db2 connect to $DB_NAME user $DB_USERID using ****" >> "$LOG"
   db2 -l "$LOG" "connect to $DB_NAME"
   #user $DB_USERID using '$DB_PASSWORD'"
   rc=$?
   #ignore warningings
   if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
      rc=0
   fi
fi

#We are connected to the database
if [ $rc -eq 0 ] 
then
   is_connected=1
   else
   is_connected=0
   rc=1
fi

#Create database objects like buffer pool, table space, XML tables, Staging tables, indices etc.
if [ $rc -eq 0 ] 
then 
   echo db2 -s -t -f $CURRENT_DIR/tklmsql.db2 >> "$LOG"
   db2 -l "$LOG" -s -t -f "$CURRENT_DIR/tklmsql.db2"
   rc=$?
   #ignore warningings
   if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
      rc=0
   fi
fi

# update logsecond to handle migration of large numbers of keys ...
if [ $rc -eq 0 ] 
then 
   echo db2 update db cfg for $DB_NAME using logsecond 25 >> "$LOG"
   db2 -l "$LOG" update db cfg for $DB_NAME using logsecond 25
   rc=$?
   #ignore warnings
   if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
      rc=0
   fi
fi
#update the logfilsiz to handle migration of 50000 EKM keys.
if [ $rc -eq 0 ] 
then 
   echo db2 update db cfg for $DB_NAME using logfilsiz 10000 >> "$LOG"
   db2 -l "$LOG" update db cfg for $DB_NAME using logfilsiz 10000
   rc=$?
   #ignore warnings
   if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
      rc=0
   fi
fi

#Disconnect from the database, no need to check for return code
if [ $is_connected -eq 1 ] ; then
   echo db2 connect reset >> "$LOG"
   db2 -l "$LOG" "connect reset"
   is_connected=0
fi


#update the SVCENAME
echo db2set DB2_REDUCED_OPTIMIZATION=ZZJN OFF >> "$LOG"
db2set DB2_REDUCED_OPTIMIZATION="ZZJN OFF" >> "$LOG"
echo db2 update dbm cfg using HEALTH_MON OFF >> "$LOG"
db2 update dbm cfg using HEALTH_MON OFF >> "$LOG"
echo db2 update dbm cfg using SVCENAME  $SVCE_NAME >> "$LOG"
db2 update dbm cfg using SVCENAME $SVCE_NAME >> "$LOG"
echo db2 update db cfg for $DB_NAME using APPLHEAPSZ 2048 >> "$LOG"
db2 update db cfg for $DB_NAME using APPLHEAPSZ 2048 >> "$LOG"
echo db2 update db cfg for $DB_NAME using LOGARCHMETH1 $dbarchdir  >> "$LOG"
db2 update db cfg for $DB_NAME using LOGARCHMETH1 DISK:$dbarchdir >> "$LOG"
echo db2 stop database manager force>> "$LOG"
db2 -l "$LOG" "stop database manager force"
echo db2 start database manager >> "$LOG"
db2 -l "$LOG" "start database manager"
echo "db2 backup db $DB_NAME to /dev/null"
db2 backup db $DB_NAME to /dev/null >> "$LOG"

#If an error has occurred and we created the database then drop the database
if [ $rc -gt 0 ] ; then
   if [ $db_created -eq 1 ] ; then
      echo db2 drop database $DB_NAME >> "$LOG"
      db2 -l "$LOG" "drop database $DB_NAME"
   fi

   if [ $inst_created -eq 1 ]
   then
	echo db2 stop database manager force >> "$LOG"
	db2 -l "$LOG" "stop database manager force"
        echo  $DBIDROP -f $DB_INST >> "$LOG"
        $DBIDROP -f $DB_INST
        
   fi
   if [ $dir_created -eq 1 ]
   then
      rm -rf $dbarchdir
   fi
fi

echo "---------------------------------------------------------------------------------------------------" >> $LOG
if [ $rc -eq 0 ]; then
echo "Return=$rc" >> $LOG
else
echo "Return $rc" >> $LOG
fi
echo "---------------------------------------------------------------------------------------------------" >> $LOG
exit $rc

