#!/bin/sh
#set -x
#
# Licensed Materials-Property of IBM
# (C)Copyright IBM Corp 2008
# All rights reserved
# US Government Users Restricted Rights
# Use, duplication, or disclosure restricted
# by GSA ADP Schedule Contract with IBM Corp
# 
# The script is to be called from the IBM Tivoli Key Life Cycle Manager 
# Installer
#
# arg1 -  log  -   log file to be used by this script 
# arg2 -  instance  -  instance name (for example db2inst2) 
# arg3 -  dbname  -  database name
# arg4 -  dbinstance_owner_home - home of the instance owner  
# arg5 -  dbbackupdir  -  directory containing backup file
# arg6 -  dbtimestamp  -  backup time stamp
# arg7 -  dbstoptime -  backup time stamp - not used in FP 1 and later versions
#  Note:  The script 
#	        must be run on the same machine where DB2 server is installed.
#		does do not parameter validation
#
#------------------------------------------------------------------------------

LOG=$1
DB_INST=$2
DB_NAME=$3
INST_HOME=$4
DB_BACKUPDIR=$5
DB_TIMESTAMP=$6
DBSTOPTIME=$7
ARCHLOGDIR=${DB_BACKUPDIR}/${DB_TIMESTAMP}_archd
rm -rf $ARCHLOGDIR
mkdir $ARCHLOGDIR
chmod a+w $ARCHLOGDIR

#The return code from the script
rc=0

#temporary return code
dbrc=0

if [ $# -ne 7 ]
then
echo Usage: dbtklmrestore.sh logfile dbinstance dbname dbinstance_owner_home dbbackupdir dbtimestamp dbstoptime
    exit 2
fi


##################################################################
echo "---------------------------------------------------------------------------------------------------" >> $LOG
echo Started executing dbtklmrestore.sh >> "$LOG"
echo "---------------------------------------------------------------------------------------------------" >> $LOG

CURRENT_DIR=`dirname $0`
if [ "$CURRENT_DIR" = "." ]; then
   CURRENT_DIR=`pwd`
fi


# set instance profile path to source profile of local instance

#source $DB_INST/db2profile
echo . $INST_HOME/sqllib/db2profile >> "$LOG" 
. $INST_HOME/sqllib/db2profile
rc=$?
   
echo db2 stop database manager force >> "$LOG"
db2 -l "$LOG" "stop database manager force"
rm -rf $INST_HOME/sklmdbarchive/*

echo db2 start database manager >> "$LOG"
db2 -l "$LOG" "start database manager"

echo "db2 list applications" >> "$LOG"
db2 "list applications" >> "$LOG"

echo "db2 drop db $DB_NAME" >> "$LOG"
db2 "drop db $DB_NAME" >> "$LOG"
rc=$?

echo "grep SQL1013N $LOG" >> "$LOG"
grep "SQL1013N" "$LOG"
dbrc=$?

if [ $dbrc -eq 0 ]; then
  rc=0
fi

if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
  rc=0
fi

if [ $rc -eq 0 ]; then
    echo "db2 restore DB $DB_NAME FROM $DB_BACKUPDIR TAKEN AT $DB_TIMESTAMP  WITHOUT PROMPTING" >> "$LOG"
    db2 "restore DB $DB_NAME FROM $DB_BACKUPDIR TAKEN AT $DB_TIMESTAMP  WITHOUT PROMPTING" >> "$LOG"
    rc=$?
    if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
        rc=0
    fi
fi

if [ $rc -eq 0 ]; then
    echo "db2 restore DB $DB_NAME logs FROM $DB_BACKUPDIR  LOGTARGET $ARCHLOGDIR " >> "$LOG"
    db2 "restore DB $DB_NAME logs FROM $DB_BACKUPDIR LOGTARGET $ARCHLOGDIR" >> "$LOG"
    rc=$?
    if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
        rc=0
    fi
fi


if [ $rc -eq 0 ]; then 
   echo db2 stop database manager force >> "$LOG"
   db2 -l "$LOG" "stop database manager force"
   echo db2 start database manager >> "$LOG"
   db2 -l "$LOG" "start database manager"

# add NORETRIEVE so that database would not look at LOGARCHMETH1 directory for additional logs.
   echo "db2 rollforward db $DB_NAME to end of logs AND COMPLETE OVERFLOW log path ($ARCHLOGDIR) NORETRIEVE" >> "$LOG"
   #echo "db2 rollforward db $DB_NAME to $DBSTOPTIME USING LOCAL TIME OVERFLOW log path ($ARCHLOGDIR) NORETRIEVE" >> "$LOG"
   #db2 "rollforward db $DB_NAME to $DBSTOPTIME USING LOCAL TIME OVERFLOW log path ($ARCHLOGDIR) NORETRIEVE" >> "$LOG"
   db2 "rollforward db $DB_NAME to end of logs AND COMPLETE  OVERFLOW log path ($ARCHLOGDIR) NORETRIEVE" >> "$LOG"
   #echo $?
  # echo "db2 rollforward db $DB_NAME complete" >> "$LOG"
   #db2 "rollforward db $DB_NAME complete" >> "$LOG"
   rc=$?
   if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
      rc=0
   fi

   if [ $rc -eq 0 ]; then  
      echo db2 stop database manager force >> "$LOG"
      db2 -l "$LOG" "stop database manager force"
      echo db2 start database manager >> "$LOG"
      db2 -l "$LOG" "start database manager"
      
      # Bind the Schema to the database. This is done in case a DB2 PTF is put on the machine that
      # does not match the DB2 version that the backup was taken on.
      echo "db2 connect to $DB_NAME" >> "$LOG"
      db2 "connect to $DB_NAME" >> "$LOG"

      echo "db2 bind '$INST_HOME/sqllib/bnd/db2schema.bnd' blocking all grant public sqlerror continue" >> "$LOG"
      db2 "bind '$INST_HOME/sqllib/bnd/db2schema.bnd' blocking all grant public sqlerror continue" >> "$LOG"

   fi

fi


rm -rf $ARCHLOGDIR
echo "---------------------------------------------------------------------------------------------------" >> $LOG
if [ $rc -eq 0 ]; then
echo "Return=$rc"
echo "Return=$rc" >> $LOG
else
echo "Return $rc" >> $LOG
fi
echo "---------------------------------------------------------------------------------------------------" >> $LOG
exit $rc

