#!/bin/sh
#set -x
#
# Licensed Materials-Property of IBM
# (C)Copyright IBM Corp 2008
# All rights reserved
# US Government Users Restricted Rights
# Use, duplication, or disclosure restricted
# by GSA ADP Schedule Contract with IBM Corp
# 
# The script is to be called from the IBM Tivoli Key Lifecycle Manager 
# Installer
#
# arg1 -  log  -   log file to be used by this script 
# arg2 -  instance  -  instance name (for example db2inst2) 
# arg3 -  dbname  -  database name
# arg4 -  dblocation - path where Db2 is installed (for example /opt/IBM/db2/V9.1
# arg5 -  dbinstance_owner_home - (For example /home/db2inst2)
#
#  Note:  The script 
#	        must be run on the same machine where DB2 server is installed.
#		does do not parameter validation
#	        will drop the database instance and database
#
#------------------------------------------------------------------------------

LOG=$1
DB_INST=$2
DB_NAME=$3
DBPATH=$4
INST_HOME=$5
CREATEINSTDB=0

#flag to indicate that we created the instance
inst_created=0
#flag to indicate that we created the Tivoli Key Lifecycle Manager database
db_created=0
#flag to indicate if we are connected to the database
is_connected=0
#The return code from the script
rc=0

if [ $# -ne 5 ]
then
echo Usage: dbtklmcr.sh logfile dbinstance dbname dblocation dbinstance_owner_home
    exit 2
fi


##################################################################
echo "---------------------------------------------------------------------------------------------------" >> $LOG
echo Started executing dbtklmrm.sh >> "$LOG"
echo "---------------------------------------------------------------------------------------------------" >> $LOG

CURRENT_DIR=`dirname $0`
if [ "$CURRENT_DIR" = "." ] ; then
   CURRENT_DIR=`pwd`
fi

DBIDROP=$DBPATH/instance/db2idrop

# set instance profile path to source profile of local instance

#source $DB_INST/db2profile
echo . $INST_HOME/sqllib/db2profile >> "$LOG" 
if [ -f  $INST_HOME/sqllib/db2profile ]; then
. $INST_HOME/sqllib/db2profile
rc=$?
if [ $rc -ne 0 ]; then
   echo The profile could not be sourced. Refer to the Tivoli Key Lifecycle Manager documentation for manually cleaning up the system.   >> $LOG
fi
else
rc=1
   echo The profile could could not be found. Refer to the Tivoli Key Lifecycle Manager documentation for manually cleaning up the system.   >> $LOG
fi

   




#If an error has occurred and we created the database then drop the database
echo db2 drop database $DB_NAME >> "$LOG"
db2 -l "$LOG" "drop database $DB_NAME"
rc=$?
if [ $rc -ne  0 ]; then
echo The database $DB_NAME could not be removed. Refer to the Tivoli Key Lifecycle Manager documentation for manually cleaning up the system.   >> $LOG
fi

if [ $rc -eq 0 ]; then
echo db2 stop database manager force >> "$LOG"

db2 -l "$LOG" "stop database manager force"
echo  $DBIDROP -f $DB_INST >> "$LOG"
$DBIDROP -f $DB_INST
rc=$?
if [ $rc -ne 0 ]; then 
echo The database instance $DB_INST could not be removed. Refer to the Tivoli Key Lifecycle Manager documentation for manually cleaning up the system.   >> $LOG
fi 
fi

echo "---------------------------------------------------------------------------------------------------" >> $LOG
if [ $rc -eq 0 ]; then
echo "Return=$rc" >> $LOG
else
echo "Return $rc" >> $LOG
fi
echo "---------------------------------------------------------------------------------------------------" >> $LOG
exit $rc

