-- disable auto-commit option
update command options using c off;

-------------------------------------- BEGIN CREATE TABLESPACES ----------------------------

CREATE BUFFERPOOL TKLMBP_LG 
  SIZE 3000
  PAGESIZE 8K;

CREATE BUFFERPOOL TKLMBP_8K_LG 
  SIZE 3000
  PAGESIZE 8K;

CREATE BUFFERPOOL TKLMBP_32K_LG 
  SIZE 3000
  PAGESIZE 32K;

CREATE BUFFERPOOL TKLMBP_SM 
  SIZE 3000
  PAGESIZE 8K;

CREATE BUFFERPOOL TKLMBP_4K_LG_IDX 
  SIZE 3000
  PAGESIZE 4K;

CREATE BUFFERPOOL TKLMBP_IDX 
  SIZE 3000
  PAGESIZE 8K;
  
CREATE BUFFERPOOL TKLMBP_32K_IDX 
  SIZE 3000
  PAGESIZE 32K;

CREATE  REGULAR  TABLESPACE TKLM_DATA_LG PAGESIZE 8 K  
  MANAGED BY DATABASE  
  USING ( FILE 'tklm_data_lg' 128M)
  AUTORESIZE YES INCREASESIZE 128M 
  EXTENTSIZE 8
  OVERHEAD 24.1 
  PREFETCHSIZE 16 
  TRANSFERRATE 0.9 
  BUFFERPOOL TKLMBP_LG;

COMMENT ON TABLESPACE TKLM_DATA_LG IS 'Tablespace used for storing large 8k tklm tables';

CREATE  REGULAR  TABLESPACE TKLM_DATA_32K_LG PAGESIZE 32 K  
  MANAGED BY DATABASE  
  USING ( FILE 'tklm_data_32k_lg' 512M)
  AUTORESIZE YES INCREASESIZE 512M 
  EXTENTSIZE 8
  OVERHEAD 24.1 
  PREFETCHSIZE 16 
  TRANSFERRATE 0.9 
  BUFFERPOOL TKLMBP_32K_LG;

COMMENT ON TABLESPACE TKLM_DATA_32K_LG IS 'Tablespace used for storing large 32k tklm tables';

CREATE  REGULAR  TABLESPACE TKLM_DATA_SM PAGESIZE 8 K  
  MANAGED BY DATABASE  
  USING ( FILE 'tklm_data_sm' 4M)
  AUTORESIZE YES INCREASESIZE 4M 
  EXTENTSIZE 8
  OVERHEAD 24.1 
  PREFETCHSIZE 16 
  TRANSFERRATE 0.9 
  BUFFERPOOL TKLMBP_SM;

COMMENT ON TABLESPACE TKLM_DATA_SM IS 'Tablespace used for storing small 8k tklm tables';

CREATE  REGULAR  TABLESPACE TKLM_4K_IDX PAGESIZE 4 K  
  MANAGED BY DATABASE  
  USING ( FILE 'tklm_4k_indices' 32M)
  AUTORESIZE YES INCREASESIZE 32M  
  EXTENTSIZE 8 
  OVERHEAD 24.1 
  PREFETCHSIZE 16 
  TRANSFERRATE 0.9 
  BUFFERPOOL TKLMBP_4K_LG_IDX;

  COMMENT ON TABLESPACE TKLM_4K_IDX IS 'Tablespace used for storing tklm 4k indices';

CREATE  REGULAR  TABLESPACE TKLM_IDX PAGESIZE 8 K  
  MANAGED BY DATABASE  
  USING ( FILE 'tklm_indices' 64M)
  AUTORESIZE YES INCREASESIZE 64M  
  EXTENTSIZE 8 
  OVERHEAD 24.1 
  PREFETCHSIZE 16 
  TRANSFERRATE 0.9 
  BUFFERPOOL TKLMBP_IDX;

  COMMENT ON TABLESPACE TKLM_IDX IS 'Tablespace used for storing tklm 8k indices';

  CREATE  REGULAR  TABLESPACE TKLM_32K_IDX PAGESIZE 32 K  
  MANAGED BY DATABASE  
  USING ( FILE 'tklm_32k_indices' 256M)
  AUTORESIZE YES INCREASESIZE 256M  
  EXTENTSIZE 8 
  OVERHEAD 24.1 
  PREFETCHSIZE 16 
  TRANSFERRATE 0.9 
  BUFFERPOOL TKLMBP_32K_IDX;

  COMMENT ON TABLESPACE TKLM_32K_IDX IS 'Tablespace used for storing tklm indices for tables in 32k tablespaces';

CREATE  SYSTEM TEMPORARY TABLESPACE TEMP_DATA PAGESIZE 8 K  
  MANAGED BY DATABASE  
  USING ( FILE 'system_data' 64M) 
  AUTORESIZE YES INCREASESIZE 64M
  EXTENTSIZE 8 
  OVERHEAD 24.1 
  PREFETCHSIZE 16 
  TRANSFERRATE 0.9 
  BUFFERPOOL TKLMBP_LG;

COMMENT ON TABLESPACE TEMP_DATA IS 'Temporary tablespace managed by database system';

----------------------------BEGIN SCHEDULER------------------------------------------------

CREATE BUFFERPOOL TKLMBP_SCH 
  SIZE 3000
  PAGESIZE 8K;

CREATE TABLESPACE TKLM_SCHEDULER PAGESIZE 8 K  
  MANAGED BY DATABASE 
  USING ( FILE 'tklm_scheduler' 32M)
  AUTORESIZE YES INCREASESIZE 32M  
  EXTENTSIZE 8 
  OVERHEAD 24.1 
  PREFETCHSIZE 16 
  TRANSFERRATE 0.9 
  BUFFERPOOL TKLMBP_SCH;

COMMENT ON TABLESPACE TKLM_DATA_SM IS 'Tablespace used for Websphere-based TKLM scheduler';

-------------------------------END SCHEDULER------------------------------------------------

--GRANT USE OF TABLESPACE TKLM_DATA_LG TO USER $user_name WITH GRANT OPTION;
--GRANT USE OF TABLESPACE TKLM_DATA_SM TO USER $user_name WITH GRANT OPTION;
--GRANT USE OF TABLESPACE TKLM_INDICES TO USER $user_name WITH GRANT OPTION;
--GRANT USE OF TABLESPACE TKLM_SCHEDULER TO USER $user_name WITH GRANT OPTION;
--GRANT CREATETAB, CONNECT,IMPLICIT_SCHEMA ON DATABASE TO USER $user_name;



-- commits the changes to DB2
commit;
-------------------------------END CREATE TABLESPACES --------------------------------------


-------------------------------------- CREATE TABLES ---------------------------------------

CREATE TABLE KMT_METADATA (
    PROPERTY_NAME VARCHAR(254) NOT NULL,
    PROPERTY_VALUE VARCHAR(1024) NOT NULL,
    LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
  )
  IN TKLM_DATA_SM INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;

--Table commented out because it is not clear if it will be used (by RBAC or otherwise)
--CREATE TABLE KMT_KMIP_ATTR_OPERATIONAL_POLICY (
--		UUID VARCHAR(64) NOT NULL,
--		POLICY_NAME VARCHAR(256) NOT NULL
--		--more columns representing policy details like a pointer to flat def file, etc 
--	)
--	IN TKLM_DATA_LG INDEX IN TKLM_IDX
--	DATA CAPTURE NONE ;
--ALTER TABLE KMT_KMIP_ATTR_OPERATIONAL_POLICY ADD CONSTRAINT PK_OP_POLICY PRIMARY KEY (UUID);
--CREATE UNIQUE INDEX IDX_OPPOL_NAME ON KMT_KMIP_ATTR_OPERATIONAL_POLICY (POLICY_NAME) MINPCTUSED 20;

CREATE TABLE KMT_KEY (
	UUID VARCHAR(64) NOT NULL,
	TYPE VARCHAR (64) NOT NULL,
	DESCRIPTION VARCHAR(256),
	ALGORITHM VARCHAR(512),
	LENGTH INT,
	CREATIONDATE TIMESTAMP,
	EXPIRATIONDATE TIMESTAMP,
--Obsolete column:
	RETIREDATE TIMESTAMP,
	DESTROYDATE TIMESTAMP,
--Obsolete column:	
	AUTOSTATECHANGE SMALLINT NOT NULL,
	CONSTRAINT CCAUTOSTATECHANGE1 CHECK (AUTOSTATECHANGE in(0,1)),
	ACTIVATIONDATE TIMESTAMP,
	COMPROMISEDATE TIMESTAMP,
	ARCHIVETIME TIMESTAMP,
--Obsolete column:	
	STATE INTEGER,
	PREVSTATE INTEGER,
	OWNER VARCHAR(128),
	HASHVALUE VARCHAR(256) FOR BIT DATA NOT NULL,
	VERSION INT NOT NULL,
	PENDING SMALLINT DEFAULT 0,
	FORMAT SMALLINT,
	--KMIP:
	LEASE_TIME INTEGER,
	CONTACT_INFO VARCHAR(1024),
	LAST_CHANGED_DATE TIMESTAMP,
	OPERATIONAL_POLICY VARCHAR(64),
	COMPROMISE_OCCURRENCE_DATE TIMESTAMP,
	PROCESS_START_DATE TIMESTAMP,
	PROTECT_STOP_DATE TIMESTAMP,
	REVOKE_REASON_CODE INTEGER,
	REVOKE_REASON_MESSAGE VARCHAR(2048),
	USAGE_VALUE DECIMAL(18,0),
	USAGE_TOTAL DECIMAL(18,0),
	USAGE_LIMIT_TYPE SMALLINT,
	CRYPTO_USAGE VARCHAR(32),
	KEY_DATA BLOB,
	 -- SKI SUPPORT
	X509EXT_SKI VARCHAR(256),
	SHA1PUBKEY_SKI VARCHAR(256),
	ORIGINAL_CREATION_DATE TIMESTAMP
  )
  IN TKLM_DATA_LG INDEX IN TKLM_IDX
  DATA CAPTURE NONE;

ALTER TABLE KMT_KEY ADD CONSTRAINT PK_KEY PRIMARY KEY (UUID);
--ALTER TABLE KMT_KEY ADD CONSTRAINT FK_OP_POLICY FOREIGN KEY (OPERATIONAL_POLICY)
--	REFERENCES KMT_KMIP_ATTR_OPERATIONAL_POLICY (UUID);

CREATE INDEX IDX_KEY_01 ON KMT_KEY (CREATIONDATE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_02 ON KMT_KEY (EXPIRATIONDATE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_03 ON KMT_KEY (ACTIVATIONDATE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_04 ON KMT_KEY (COMPROMISEDATE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_05 ON KMT_KEY (DESTROYDATE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_06 ON KMT_KEY (HASHVALUE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_07 ON KMT_KEY (PENDING) MINPCTUSED 20;
CREATE INDEX IDX_KEY_08 ON KMT_KEY (ALGORITHM) MINPCTUSED 20;
CREATE INDEX IDX_KEY_09 ON KMT_KEY (LENGTH) MINPCTUSED 20;
CREATE INDEX IDX_KEY_10 ON KMT_KEY (LEASE_TIME) MINPCTUSED 20;
CREATE INDEX IDX_KEY_11 ON KMT_KEY (OWNER) MINPCTUSED 20;
CREATE INDEX IDX_KEY_12 ON KMT_KEY (REVOKE_REASON_CODE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_13 ON KMT_KEY (PROCESS_START_DATE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_14 ON KMT_KEY (PROTECT_STOP_DATE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_15 ON KMT_KEY (USAGE_VALUE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_16 ON KMT_KEY (USAGE_TOTAL) MINPCTUSED 20;
CREATE INDEX IDX_KEY_17 ON KMT_KEY (USAGE_LIMIT_TYPE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_18 ON KMT_KEY (CRYPTO_USAGE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_19 ON KMT_KEY (PREVSTATE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_20 ON KMT_KEY (TYPE) MINPCTUSED 20;
CREATE INDEX IDX_KEY_21 ON KMT_KEY (X509EXT_SKI) MINPCTUSED 20;
CREATE INDEX IDX_KEY_22 ON KMT_KEY (SHA1PUBKEY_SKI) MINPCTUSED 20;
CREATE INDEX IDX_KEY_23 ON KMT_KEY (ORIGINAL_CREATION_DATE) MINPCTUSED 20;


CREATE TABLE KMT_CERT (
	UUID VARCHAR(64) NOT NULL,
	DESCRIPTION VARCHAR(256),
	ISSUERNAME VARCHAR(512),
	SERIALNUMBER VARCHAR(64),
	SUBJECTNAME VARCHAR(512),
--Obsolete column:	
	TARGETUUID VARCHAR(64),
	CREATIONDATE TIMESTAMP,
	ACTIVATIONDATE TIMESTAMP,
	EXPIRATIONDATE TIMESTAMP,        
	DESTROYDATE TIMESTAMP,
	COMPROMISEDATE TIMESTAMP,
--Obsolete column:	
	RETIREDATE TIMESTAMP,
--Obsolete column:	
	AUTOSTATECHANGE SMALLINT NOT NULL,
--Obsolete column:	
	STATE INTEGER,
	PREVSTATE INTEGER,
	OWNER VARCHAR(128),
	ARCHIVETIME TIMESTAMP,
	HASHVALUE VARCHAR(256) FOR BIT DATA NOT NULL,
	VERSION INT NOT NULL,
	NOTIFY SMALLINT NOT NULL DEFAULT 1,
	TRUSTED SMALLINT NOT NULL DEFAULT 1,
	--KMIP:
	LEASE_TIME INTEGER,
	CONTACT_INFO VARCHAR(1024),
	LAST_CHANGED_DATE TIMESTAMP,
	COMPROMISE_OCCURRENCE_DATE TIMESTAMP,
	OPERATIONAL_POLICY VARCHAR(64),
	CERTIFICATE_TYPE INTEGER,
	PROCESS_START_DATE TIMESTAMP,
	PROTECT_STOP_DATE TIMESTAMP,
	REVOKE_REASON_CODE INTEGER,
	REVOKE_REASON_MESSAGE VARCHAR(2048),
	CONSTRAINT NOTIFY1 CHECK (NOTIFY in(0,1)),
	USAGE_VALUE DECIMAL(18,0),
	USAGE_TOTAL DECIMAL(18,0),
	USAGE_LIMIT_TYPE SMALLINT,
	KEY_DATA BLOB,
	PENDING_TRUST SMALLINT DEFAULT 0,
	PENDING_TRUST_DATE TIMESTAMP,
--New for V3 Cert support:
    DIGITAL_SIGNATURE_ALGORITHM SMALLINT NOT NULL,
    CERTIFICATE_LENGTH INTEGER NOT NULL,
    -- SKI SUPPORT
    X509EXT_SKI VARCHAR(256),
    SHA1PUBKEY_SKI VARCHAR(256),
	CRYPTO_USAGE VARCHAR(32),
	ALGORITHM VARCHAR(512),
	LENGTH INT,
    ORIGINAL_CREATION_DATE TIMESTAMP
  )
  IN TKLM_DATA_LG INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;

ALTER TABLE KMT_CERT ADD CONSTRAINT PK_CERT PRIMARY KEY (UUID);
--ALTER TABLE KMT_CERT ADD CONSTRAINT FK_OP_POLICY FOREIGN KEY (OPERATIONAL_POLICY)
--	REFERENCES KMT_KMIP_ATTR_OPERATIONAL_POLICY (UUID);
CREATE INDEX IDX_CERT_01 ON KMT_CERT (CREATIONDATE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_02 ON KMT_CERT (EXPIRATIONDATE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_03 ON KMT_CERT (ACTIVATIONDATE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_04 ON KMT_CERT (COMPROMISEDATE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_05 ON KMT_CERT (DESTROYDATE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_06 ON KMT_CERT (HASHVALUE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_07 ON KMT_CERT (NOTIFY) MINPCTUSED 20;
CREATE INDEX IDX_CERT_08 ON KMT_CERT (TRUSTED) MINPCTUSED 20;
CREATE INDEX IDX_CERT_09 ON KMT_CERT (LEASE_TIME) MINPCTUSED 20;
CREATE INDEX IDX_CERT_10 ON KMT_CERT (SERIALNUMBER) MINPCTUSED 20;
CREATE INDEX IDX_CERT_11 ON KMT_CERT (ISSUERNAME) MINPCTUSED 20;
CREATE INDEX IDX_CERT_12 ON KMT_CERT (SUBJECTNAME) MINPCTUSED 20;
CREATE INDEX IDX_CERT_13 ON KMT_CERT (OWNER) MINPCTUSED 20;
CREATE INDEX IDX_CERT_14 ON KMT_CERT (REVOKE_REASON_CODE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_15 ON KMT_CERT (PROCESS_START_DATE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_16 ON KMT_CERT (PROTECT_STOP_DATE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_17 ON KMT_CERT (USAGE_VALUE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_18 ON KMT_CERT (USAGE_TOTAL) MINPCTUSED 20;
CREATE INDEX IDX_CERT_19 ON KMT_CERT (USAGE_LIMIT_TYPE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_20 ON KMT_CERT (PREVSTATE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_21 ON KMT_CERT (DIGITAL_SIGNATURE_ALGORITHM) MINPCTUSED 20;
CREATE INDEX IDX_CERT_22 ON KMT_CERT (X509EXT_SKI) MINPCTUSED 20;
CREATE INDEX IDX_CERT_23 ON KMT_CERT (SHA1PUBKEY_SKI) MINPCTUSED 20;
CREATE INDEX IDX_CERT_24 ON KMT_CERT (ORIGINAL_CREATION_DATE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_25 ON KMT_CERT (CRYPTO_USAGE) MINPCTUSED 20;
CREATE INDEX IDX_CERT_26 ON KMT_CERT (ALGORITHM) MINPCTUSED 20;
CREATE INDEX IDX_CERT_27 ON KMT_CERT (LENGTH) MINPCTUSED 20;

CREATE TABLE KMT_POLICY (
    UUID VARCHAR(64) NOT NULL,
    MANADATORY SMALLINT NOT NULL,
    POLICYTYPE SMALLINT NOT NULL,
    POLICY  VARCHAR (3072) NOT NULL,
    VERSION INT NOT NULL,
    LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
  ) 
  IN TKLM_DATA_SM INDEX IN TKLM_IDX
  DATA CAPTURE NONE;

ALTER TABLE KMT_POLICY ADD CONSTRAINT PK_POLICY PRIMARY KEY(UUID);

CREATE TABLE KMT_KEYPOLICY_RN (
    UUID VARCHAR(64) NOT NULL,
    POLICYID VARCHAR(64) NOT NULL,
    LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
  )
  IN TKLM_DATA_LG INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;



ALTER TABLE KMT_KEYPOLICY_RN ADD CONSTRAINT PK_KEYPOLICYRN PRIMARY KEY(POLICYID, UUID);



CREATE TABLE KMT_CERPOLICY_RN (
    UUID VARCHAR(64) NOT NULL,
    POLICYID VARCHAR(64) NOT NULL,
    LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
  )
  IN TKLM_DATA_LG INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;

ALTER TABLE KMT_CERPOLICY_RN ADD CONSTRAINT PK_CERPOLICYRN  PRIMARY KEY (POLICYID, UUID);



-- NOT NEEDED FOR V1
--CREATE TABLE KMT_EXKEYATT (
--    UUID VARCHAR(64) NOT NULL,
--    NAME VARCHAR(256) NOT NULL,
--    VALUE VARCHAR(1020),
--    ATTRDATATYPE SMALLINT
--  )
--IN TKLM_DATA_LG INDEX IN TKLM_IDX
--   DATA CAPTURE NONE ;



CREATE TABLE KMT_KEYSTR (
    UUID VARCHAR(64) NOT NULL,
    KEYSTORELOCATION VARCHAR(2048) NOT NULL,
    KEYSTOREPWD VARCHAR(256),
    KEYSTORETYPE VARCHAR(64),
    KEYSTORENAME VARCHAR(64) NOT NULL,
    USERID    VARCHAR (256),
    VERSION INT NOT NULL,
    LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
  )
  IN TKLM_DATA_SM INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;


ALTER TABLE KMT_KEYSTR ADD CONSTRAINT PK_KEYSTR PRIMARY KEY (UUID);
CREATE UNIQUE INDEX IDX_KEYSTR_01 ON KMT_KEYSTR(KEYSTORENAME);

INSERT INTO KMT_KEYSTR(UUID,KEYSTORELOCATION,KEYSTOREPWD,KEYSTORETYPE,KEYSTORENAME,VERSION) VALUES
('DUMMY-KEYSTORE-1','DummyLoc','DummyPwd','DummyType','defaultKeyStore',1);


CREATE TABLE KMT_KEYSTR_RN (
    UUID VARCHAR(64) NOT NULL,
    KEYSTOREID VARCHAR(64) NOT NULL,
    ALIAS VARCHAR(256) NOT NULL,
    KEYPASSWORD VARCHAR(256)
  )
  IN TKLM_DATA_LG INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;

ALTER TABLE KMT_KEYSTR_RN ADD CONSTRAINT PK_KEYSTR_RN PRIMARY KEY (UUID, KEYSTOREID, ALIAS);
ALTER TABLE KMT_KEYSTR_RN ADD CONSTRAINT KEYSTRRN_FK1 FOREIGN KEY (KEYSTOREID)
  REFERENCES KMT_KEYSTR (UUID) ON DELETE RESTRICT;
ALTER TABLE KMT_KEYSTR_RN ADD CONSTRAINT KEYSTRRN_FK2 FOREIGN KEY (UUID)
  REFERENCES KMT_KEY (UUID) ON DELETE CASCADE;
CREATE INDEX IDX_KEYSTRRN_01 ON KMT_KEYSTR_RN(ALIAS) MINPCTUSED 20;
CREATE INDEX IDX_KEYSTRRN_02 ON KMT_KEYSTR_RN(UUID) MINPCTUSED 20;



CREATE TABLE KMT_CERTSTR_RN (
    UUID VARCHAR(64) NOT NULL,
    KEYSTOREID VARCHAR(64) NOT NULL,
    ALIAS VARCHAR(256) NOT NULL
  )
  IN TKLM_DATA_LG INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;

ALTER TABLE KMT_CERTSTR_RN ADD CONSTRAINT PK_CERTSTR_RN PRIMARY KEY (UUID, KEYSTOREID, ALIAS);

ALTER TABLE KMT_CERTSTR_RN ADD CONSTRAINT CERTSTRRN_FK1 FOREIGN KEY (KEYSTOREID)
  REFERENCES KMT_KEYSTR (UUID)
  ON DELETE RESTRICT;

CREATE UNIQUE INDEX IDX_CERTSTRRN_01 ON KMT_CERTSTR_RN(KEYSTOREID, ALIAS);
CREATE INDEX IDX_CERTSTRRN_02 ON KMT_CERTSTR_RN(UUID);


ALTER TABLE KMT_CERTSTR_RN ADD CONSTRAINT CERTSTRRN_FK2 FOREIGN KEY (UUID)
  REFERENCES KMT_CERT (UUID)
  ON DELETE CASCADE;

-- table store data about groupings of TKLM entities.
--GROUPTYPE: 1=key, 2=device
CREATE TABLE KMT_GROUP (
    UUID VARCHAR(64) NOT NULL,
    GROUPTYPE SMALLINT NOT NULL,
    GROUPNAME VARCHAR(256) NOT NULL,
    INITIALIZATIONDATE TIMESTAMP,
    ACTIVESTATEDURATION INT,
    ACTIVATIONDATE TIMESTAMP,
    LASTACTIVEMEMBER VARCHAR(256), 
    KEYCOUNT INT DEFAULT 0, 
    KEYUNUSED INT DEFAULT 0, 
    VERSION INT NOT NULL
  )
  IN TKLM_DATA_LG INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;

ALTER TABLE KMT_GROUP ADD CONSTRAINT PK_GROUP PRIMARY KEY (UUID);
CREATE UNIQUE INDEX IDX_GROUP_01 ON KMT_GROUP(GROUPNAME, GROUPTYPE);
CREATE INDEX IDX_GROUP_02 ON KMT_GROUP(KEYCOUNT);
CREATE INDEX IDX_GROUP_03 ON KMT_GROUP(KEYUNUSED);
CREATE INDEX IDX_GROUP_04 ON KMT_GROUP(GROUPTYPE);
CREATE INDEX IDX_GROUP_05 ON KMT_GROUP(LASTACTIVEMEMBER);



-- table stores data about group memebers
--MEMBERTYPE: 0=subgroup, 1=key, 2=certificate, 3=device
CREATE TABLE KMT_GROUP_MEMBERS (
    GROUPID VARCHAR(64) NOT NULL,
    MEMBERID  VARCHAR(64) NOT NULL,
    MEMBERTYPE  SMALLINT NOT NULL,
    MEMSEQNO INT GENERATED ALWAYS AS IDENTITY (START WITH -2147483648, INCREMENT BY 1),
    LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
  )
  IN TKLM_DATA_LG INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;

ALTER TABLE KMT_GROUP_MEMBERS ADD CONSTRAINT PK_GR_MEMB_ID PRIMARY KEY (GROUPID, MEMBERID);
ALTER TABLE KMT_GROUP_MEMBERS ADD CONSTRAINT GROUP_MEMBERS_FK1 FOREIGN KEY (GROUPID)
  REFERENCES KMT_GROUP (UUID) ON DELETE CASCADE;
CREATE INDEX IDX_KMT_GROUP_MEMBERS_01 ON KMT_GROUP_MEMBERS(GROUPID) MINPCTUSED 20;
CREATE INDEX IDX_KMT_GROUP_MEMBERS_02 ON KMT_GROUP_MEMBERS(MEMBERID) MINPCTUSED 20;
CREATE INDEX IDX_KMT_GROUP_MEMBERS_03 ON KMT_GROUP_MEMBERS(MEMSEQNO) MINPCTUSED 20;
CREATE INDEX IDX_KMT_GROUP_MEMBERS_04 ON KMT_GROUP_MEMBERS(MEMBERTYPE) MINPCTUSED 20;





-- NOT A V1
-- CREATE TABLE KMT_EXCERTATT (
--    UUID VARCHAR(64) NOT NULL,
--    NAME VARCHAR(256) NOT NULL,
--    VALUE VARCHAR(1020),
--    ATTRDATATYPE SMALLINT
--  )
--  DATA CAPTURE NONE ;

CREATE TABLE KMT_TEMPL (
    UUID VARCHAR(64) NOT NULL,
    TEMPLNAME VARCHAR(256),
    TYPE VARCHAR(64),
    APPNAME VARCHAR(256),
    TEMPL VARCHAR(7072),
    VERSION INT NOT NULL,
    LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
  )
  IN TKLM_DATA_SM INDEX IN TKLM_IDX
  DATA CAPTURE NONE;

ALTER TABLE KMT_TEMPL ADD CONSTRAINT PK_TEMPL PRIMARY KEY(UUID);

CREATE INDEX IDX_TEMPL_01 on KMT_TEMPL(TEMPLNAME) MINPCTUSED 20;
CREATE INDEX IDX_TEMPL_02 on KMT_TEMPL(TYPE) MINPCTUSED 20;
CREATE INDEX IDX_TEMPL_03 On KMT_TEMPL(APPNAME) MINPCTUSED 20;

-- table store metadata about devices.
-- 0 for UNKNOWN, 1 - JAG 2 - LTO-4
-- no known attribute is unique
CREATE TABLE KMT_DEVICE (
    UUID VARCHAR(64) NOT NULL,
    DEVICETYPE SMALLINT,
    SERIALNO VARCHAR(64) NOT NULL,
    WWN VARCHAR(128) NOT NULL,
    SYMALIAS VARCHAR(256),
    DEVICECERT VARCHAR(2048) FOR BIT DATA,
    KEYALIAS1 VARCHAR(256),
    KEYALIAS2 VARCHAR(256),
    DESCRIPTION VARCHAR(256),
    VERSION INT NOT NULL,
	DEVICETEXT VARCHAR(256),
	DEVICEPENDING SMALLINT,
	DEVICEPENDINGDATE TIMESTAMP,
	ORIGDEVICETYPE SMALLINT,
	LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP 
  )
  IN TKLM_DATA_LG INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;

ALTER TABLE KMT_DEVICE ADD CONSTRAINT PK_DEV PRIMARY KEY (UUID);
--CREATE UNIQUE INDEX IDX_DEVICE_01 On KMT_DEVICE(SERIALNO, DEVICETYPE, WWN) MINPCTUSED 20;
CREATE UNIQUE INDEX IDX_DEVICE_01 On KMT_DEVICE(SERIALNO) MINPCTUSED 20;


CREATE TABLE KMT_DEVAUDIT (
    UUID VARCHAR(64) NOT NULL, 
    SERIALNO VARCHAR(64),
    VOLSERIALNO VARCHAR(128),
    WWN VARCHAR(128),
    KEYALIAS1 VARCHAR(256),
    KEYALIAS2 VARCHAR(256),
    TIME_STAMP TIMESTAMP,
    DKI VARCHAR(128),
    EXTATTR VARCHAR(2800)
  )
  IN TKLM_DATA_LG INDEX IN TKLM_IDX
  DATA CAPTURE NONE ;


ALTER TABLE KMT_DEVAUDIT ADD CONSTRAINT PK_DEVAUDIT PRIMARY KEY (UUID);


INSERT INTO KMT_METADATA
(property_name,property_value)
VALUES
('SchemaMajorVersion','2');

INSERT INTO KMT_METADATA
(property_name,property_value)
VALUES
('SchemaMinorVersion','0');

INSERT INTO KMT_METADATA
(property_name,property_value)
VALUES
('SchemaPtfLevel','0');


--DELETE FROM KMT_TEMPL;

INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000001', 'KLM_CERTIFICATE_METADATA_DEFAULTS', 'CERT_TEMPLATE', '', '<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KLM_CERTIFICATE_METADATA_DEFAULTS">
  <Element ElementID="APPLICATION_NAME" ElementValue="TS1120ENCRYPTION"></Element>
      <Element ElementID="ACTIVATION_START_IN_DAYS" ElementValue="0"></Element>
      <Element ElementID="RETIREMENT_NUMBER_OF_DAYS_AFTER_ACTIVATION" ElementValue="7300"></Element>
</TKLMTemplate>',0);


INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000002', 'UI_CONFIG_CERIFICATE_PANEL',
'UI_CONFIG_CERIFICATE_PANEL', '', 

'<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="UI_CONFIG_CERTIFICATE_PANEL">
  
  <Element ElementID="CERT_RADIO_DEFAULT" ElementValue="SELF_SIGNED"></Element>
    
  <Element ElementID="SET_DEFAULT_CERT_ENABLED" ElementValue="true"></Element>
    
  <Element ElementID="DEFAULT_FILE_CERT_LOCATION" ElementValue="C:/Certs/"></Element>
    
  <Element ElementID="SELF_SIGNED_RADIO_ENABLED" ElementValue="true"></Element>
  <Element ElementID="CERT_REQ_RADIO_ENABLED" ElementValue="true"></Element>
  <Element ElementID="KS_CERT_RADIO_ENABLED" ElementValue="true"></Element>
  <Element ElementID="FILE_CERT_RADIO_ENABLED" ElementValue="true"></Element>
    
  <Element ElementID="KS_ALIAS_ENABLED" ElementValue="true"></Element>
  <Element ElementID="KS_ALIAS_VISIBLE" ElementValue="true"></Element>
  <Element ElementID="KS_ALIAS_DEFAULT" ElementValue=""></Element>
    
  <Element ElementID="CN_ENABLED" ElementValue="true"></Element>
  <Element ElementID="CN_VISIBLE" ElementValue="true"></Element>
  <Element ElementID="CN_DEFAULT" ElementValue=""></Element>
    
  <Element ElementID="OU_ENABLED" ElementValue="true"></Element>
  <Element ElementID="OU_VISIBLE" ElementValue="true"></Element>
  <Element ElementID="OU_DEFAULT" ElementValue=""></Element>
  
  <Element ElementID="ORG_ENABLED" ElementValue="true"></Element>
  <Element ElementID="ORG_VISIBLE" ElementValue="true"></Element>
  <Element ElementID="ORG_DEFAULT" ElementValue=""></Element>
  
  <Element ElementID="C_ALIAS_ENABLED" ElementValue="true"></Element>
  <Element ElementID="C_ALIAS_VISIBLE" ElementValue="true"></Element>
  <Element ElementID="C_ALIAS_DEFAULT" ElementValue=""></Element>
    
  <Element ElementID="S_ALIAS_ENABLED" ElementValue="true"></Element>
  <Element ElementID="S_ALIAS_VISIBLE" ElementValue="true"></Element>
  <Element ElementID="S_ALIAS_DEFAULT" ElementValue=""></Element>

  <Element ElementID="CO_ALIAS_ENABLED" ElementValue="true"></Element>
  <Element ElementID="CO_ALIAS_VISIBLE" ElementValue="true"></Element>
  <Element ElementID="CO_ALIAS_DEFAULT" ElementValue=""></Element>
    
  <Element ElementID="VALIDITY_ENABLED" ElementValue="true"></Element>
  <Element ElementID="VALIDITY_VISIBLE" ElementValue="true"></Element>
  <Element ElementID="VALIDITY_DEFAULT" ElementValue="1095"></Element>
  <Element ElementID="VALIDITY_MIN" ElementValue="365"></Element>
  <Element ElementID="VALIDITY_MAX" ElementValue="9999"></Element>
          
</TKLMTemplate>',0);

INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000003', 'KLM_KEY_METADATA_DEFAULTS', 'KEY_TEMPLATE', '', '<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KLM_KEY_METADATA_DEFAULTS">
  <Element ElementID="APPLICATION_NAME" ElementValue="TS1120ENCRYPTION"></Element>
      <Element ElementID="ACTIVATION_START_IN_DAYS" ElementValue="0"></Element>
      <Element ElementID="RETIREMENT_NUMBER_OF_DAYS_AFTER_ACTIVATION" ElementValue="7300"></Element>
      <Element ElementID="Algorithm" ElementValue="RSA"></Element>
      <Element ElementID="KeyType" ElementValue="SYMMETRIC_KEY"></Element>
      <Element ElementID="KeySize"  ElementValue="2048"></Element>
</TKLMTemplate>',0);

INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000004', 'UI_CONFIG_AUDIT_DEBUG_PANEL',
'UI_CONFIG_AUDIT_DEBUG_PANEL', '', 

'<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="UI_CONFIG_AUDIT_DEBUG_PANEL">

  <Element ElementID="AUDIT_LEVEL_DEFAULT" ElementValue="1"></Element>
  <Element ElementID="DEBUG_LEVEL_DEFAULT" ElementValue="0"></Element>
</TKLMTemplate>',0);

INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000005', 'UI_CONFIG_KEYSERVING_PARAMS_PANEL',
'UI_CONFIG_KEYSERVING_PARAMS_PANEL', '', 

'<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="UI_CONFIG_KEYSERVING_PARAMS_PANEL">
  
  <Element ElementID="TCP_PORT_DEFAULT" ElementValue="3801"></Element>
  <Element ElementID="TCP_TIMEOUT_DEFAULT" ElementValue="10"></Element>
  <Element ElementID="SSL_PORT_DEFAULT" ElementValue="441"></Element>
  <Element ElementID="KMIP_SSL_PORT_DEFAULT" ElementValue="5696"></Element>
  <Element ElementID="SSL_TIMEOUT_DEFAULT" ElementValue="10"></Element>
  <Element ElementID="ENABLE_ZOS_KEY_DEFAULT" ElementValue="false"></Element>
</TKLMTemplate>',0);



INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000006', 
'KEYSTORE_JCEKS', 
'UI_KEYSTORE_PANEL', '', 
'<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KEYSTORE_JCEKS">
  <Element ElementID="KEYSTORE_ALIAS_DEFAULT" ElementValue="defaultKeyStore"></Element>
</TKLMTemplate>',0);


INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000007', 
'KEYSTORE_JCERACFKS', 
'UI_KEYSTORE_PANEL', '', 
'<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KEYSTORE_JCERACFKS">
  <Element ElementID="KEYSTORE_ALIAS_DEFAULT" ElementValue="defaultKeyStore"></Element>
  <Element ElementID="HARDWARE_PROTECTION_DEFAULT" ElementValue="true"></Element>
</TKLMTemplate>',0);

INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000008', 
'KEYSTORE_JCECCARACFKS', 
'UI_KEYSTORE_PANEL', '', 
'<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KEYSTORE_JCECCARACFKS">
  <Element ElementID="KEYSTORE_ALIAS_DEFAULT" ElementValue="defaultKeyStore"></Element>
  <Element ElementID="HARDWARE_PROTECTION_DEFAULT" ElementValue="true"></Element>
</TKLMTemplate>', 0);

INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000009', 
'KEYSTORE_JCECCAKS', 
'UI_KEYSTORE_PANEL', '', 
'<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KEYSTORE_JCECCAKS">
  <Element ElementID="KEYSTORE_ALIAS_DEFAULT" ElementValue="defaultKeyStore"></Element>
  <Element ElementID="HARDWARE_PROTECTION_DEFAULT" ElementValue="true"></Element>
</TKLMTemplate>',0);

INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000010', 
'KLM_SYMMETRICKEY_AES_METADATA_DEFAULTS', 
'KEY_TEMPLATE', '', 
'<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KLM_SYMMETRICKEY_AES_METADATA_DEFAULTS">
  <Element ElementID="APPLICATION_NAME" ElementValue="TS1120ENCRYPTION"></Element>
      <Element ElementID="ACTIVATION_START_IN_DAYS" ElementValue="0"></Element>
      <Element ElementID="RETIREMENT_NUMBER_OF_DAYS_AFTER_ACTIVATION" ElementValue="7300"></Element>
      <Element ElementID="Algorithm" ElementValue="AES"></Element>
      <Element ElementID="KeyType" ElementValue="SYMMETRIC_KEY"></Element>
      <Element ElementID="KeySize"  ElementValue="256"></Element>
</TKLMTemplate>',0);

INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000011', 
'KLM_SYMMETRICKEY_DESede_METADATA_DEFAULTS', 
'KEY_TEMPLATE', '', 
'<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KLM_SYMMETRICKEY_DESede_METADATA_DEFAULTS">
  <Element ElementID="APPLICATION_NAME" ElementValue="TS1120ENCRYPTION"></Element>
      <Element ElementID="ACTIVATION_START_IN_DAYS" ElementValue="0"></Element>
      <Element ElementID="RETIREMENT_NUMBER_OF_DAYS_AFTER_ACTIVATION" ElementValue="7300"></Element>
      <Element ElementID="Algorithm" ElementValue="TRIPLE_DES"></Element>
      <Element ElementID="KeyType" ElementValue="SYMMETRIC_KEY"></Element>
      <Element ElementID="KeySize"  ElementValue="168"></Element>
</TKLMTemplate>',0);


INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000012', 'KLM_CERTIFICATE_METADATA_SPIKE', 'CERT_TEMPLATE', '', '<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KLM_CERTIFICATE_METADATA_SPIKE">
  <Element ElementID="APPLICATION_NAME" ElementValue="SPIKE"></Element>
      <Element ElementID="ACTIVATION_START_IN_DAYS" ElementValue="0"></Element>
      <Element ElementID="RETIREMENT_NUMBER_OF_DAYS_AFTER_ACTIVATION" ElementValue="7300"></Element>
</TKLMTemplate>',0);


INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000013', 'KLM_KEY_METADATA_SPIKE', 'KEY_TEMPLATE', '', '<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KLM_KEY_METADATA_SPIKE">
  <Element ElementID="APPLICATION_NAME" ElementValue="SPIKE"></Element>
      <Element ElementID="ACTIVATION_START_IN_DAYS" ElementValue="0"></Element>
      <Element ElementID="RETIREMENT_NUMBER_OF_DAYS_AFTER_ACTIVATION" ElementValue="7300"></Element>
      <Element ElementID="Algorithm" ElementValue="ECDSA"></Element>
      <Element ElementID="KeyType" ElementValue="SYMMETRIC_KEY"></Element>
      <Element ElementID="KeySize"  ElementValue="521"></Element>
</TKLMTemplate>',0);


INSERT INTO KMT_TEMPL (UUID, TEMPLNAME, TYPE, APPNAME, TEMPL, VERSION) VALUES('00000000-0000-0000-0000-000000000014', 'KLM_EC_KEY_METADATA_DEFAULTS', 'KEY_TEMPLATE', '', '<?xml version="1.0" encoding="UTF-8"?>
<TKLMTemplate TemplateName="KLM_KEY_METADATA_DEFAULTS">
  <Element ElementID="APPLICATION_NAME" ElementValue="TS1120ENCRYPTION"></Element>
      <Element ElementID="ACTIVATION_START_IN_DAYS" ElementValue="0"></Element>
      <Element ElementID="RETIREMENT_NUMBER_OF_DAYS_AFTER_ACTIVATION" ElementValue="7300"></Element>
      <Element ElementID="Algorithm" ElementValue="ECDSA"></Element>
      <Element ElementID="KeyType" ElementValue="ASYMMETRIC_KEY"></Element>
      <Element ElementID="KeySize"  ElementValue="384"></Element>
</TKLMTemplate>',0);


----------------------------BEGIN SCHEDULER------------------------------------------------

CREATE TABLE "KMT_SCH_TASK" ("TASKID" DECIMAL(19) NOT NULL ,
              "VERSION" VARCHAR(5) NOT NULL ,
              "ROW_VERSION" INTEGER NOT NULL ,
              "TASKTYPE" INTEGER NOT NULL ,
              "TASKSUSPENDED" SMALLINT NOT NULL ,
              "CANCELLED" SMALLINT NOT NULL ,
              "NEXTFIRETIME" DECIMAL(19) NOT NULL ,
              "STARTBYINTERVAL" VARCHAR(254) ,
              "STARTBYTIME" DECIMAL(19) ,
              "VALIDFROMTIME" DECIMAL(19) ,
              "VALIDTOTIME" DECIMAL(19) ,
              "REPEATINTERVAL" VARCHAR(254) ,
              "MAXREPEATS" INTEGER NOT NULL ,
              "REPEATSLEFT" INTEGER NOT NULL ,
              "TASKINFO" BLOB(102400) LOGGED NOT COMPACT,
              "NAME" VARCHAR(1024) NOT NULL ,
              "AUTOPURGE" INTEGER NOT NULL ,
              "FAILUREACTION" INTEGER ,
              "MAXATTEMPTS" INTEGER ,
              "QOS" INTEGER ,
              "PARTITIONID" INTEGER ,
              "OWNERTOKEN" VARCHAR(200) NOT NULL ,
              "CREATETIME" DECIMAL(19) NOT NULL) IN "TKLM_SCHEDULER" INDEX IN "TKLM_IDX";

ALTER TABLE "KMT_SCH_TASK" ADD PRIMARY KEY ("TASKID");

CREATE INDEX "KMT_SCH_TASK_IDX1" ON "KMT_SCH_TASK" ("TASKID",
              "OWNERTOKEN") ;

CREATE INDEX "KMT_SCH_TASK_IDX2" ON "KMT_SCH_TASK" ("NEXTFIRETIME" ASC,
               "REPEATSLEFT",
               "PARTITIONID") CLUSTER;

CREATE INDEX "KMT_SCH_TASK_IDX3" ON "KMT_SCH_TASK"("NAME") MINPCTUSED 20;

CREATE TABLE "KMT_SCH_TREG" ("REGKEY" VARCHAR(254) NOT NULL ,
              "REGVALUE" VARCHAR(254) ) IN "TKLM_SCHEDULER" INDEX IN "TKLM_IDX";

ALTER TABLE "KMT_SCH_TREG" ADD PRIMARY KEY ("REGKEY");

CREATE TABLE "KMT_SCH_LMGR" (LEASENAME VARCHAR(254) NOT NULL,
               LEASEOWNER VARCHAR(254) NOT NULL,
               LEASE_EXPIRE_TIME  DECIMAL(19),
              DISABLED VARCHAR(5))IN "TKLM_SCHEDULER" INDEX IN "TKLM_IDX";

ALTER TABLE "KMT_SCH_LMGR" ADD PRIMARY KEY ("LEASENAME");

CREATE TABLE "KMT_SCH_LMPR" (LEASENAME VARCHAR(254) NOT NULL,
              NAME VARCHAR(254) NOT NULL,
              VALUE VARCHAR(254) NOT NULL)IN "TKLM_SCHEDULER"  INDEX IN "TKLM_IDX";

CREATE INDEX "KMT_SCH_LMPR_IDX1" ON "KMT_SCH_LMPR" (LEASENAME,
               NAME);

------------------------------ END SCHEDULER -----------------------------------------------

------------------------------- START V2 UPDATES -------------------------------------------------

------------------------------- START DEVICE TYPE RELATED UPDATES --------------------------------
CREATE TABLE KMT_DEVICE_TYPE (
		BASE_FAMILY_ID SMALLINT NOT NULL,
		NAME VARCHAR(16) NOT NULL,
		ROLE_NAME VARCHAR(16),
		VERSION INTEGER NOT NULL,
		UUID SMALLINT NOT NULL GENERATED BY DEFAULT AS IDENTITY (START WITH 10000 INCREMENT BY 1 MINVALUE 1 MAXVALUE 32767 CYCLE CACHE 20),
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_IDX
	DATA CAPTURE NONE ;

CREATE INDEX DEVTYPE_BASE_FAMILY_IDX ON KMT_DEVICE_TYPE (BASE_FAMILY_ID ASC, UUID ASC) PCTFREE 0;

CREATE INDEX DEVTYPE_ROLE_IDX ON KMT_DEVICE_TYPE (ROLE_NAME ASC, UUID ASC) PCTFREE 0;

ALTER TABLE KMT_DEVICE_TYPE ADD CONSTRAINT DEVICE_TYPE_PK PRIMARY KEY (UUID);

ALTER TABLE KMT_DEVICE_TYPE ADD CONSTRAINT DEVTYPE_NAME_UN UNIQUE (NAME);

INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 0, 0, 0, 'UNKNOWN', null);
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 1, 1, 0, '3592', 'TS3592');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 2, 2, 0, 'LTO', 'LTO');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 3, 3, 0, 'DS8000', 'DS8000');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 4, 4, 0, 'DS5000', 'DS5000');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 5, 5, 0, 'GENERIC', 'GENERIC');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 6, 0, 0, 'UNSET', null);
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 7, 0, 0, 'SSLSERVER', 'CONFIG');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 8, 0, 0, 'SSLCLIENT', 'CONFIG');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 9, 0, 0, 'IKEV2SERVER', 'CONFIG');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 10, 0, 0, 'IKEV2CLIENT', 'CONFIG');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 11, 0, 0, 'CONFLICTED', null);
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 1000, 2, 0, 'BRCD_ENCRYPTOR', 'BRCD_ENCRYPTOR');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 1001, 4, 0, 'ONESECURE', 'ONESECURE');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 1002, 4, 0, 'ETERNUS_DX', 'ETERNUS_DX');
-- Commenting this for V2.5 GA. Add/uncomment this when needed
--INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
--ROLE_NAME) VALUES( 1003, 4, 0, 'IBM_SYSTEM_X_SED', 'IBM_SYSTEM_X_SED');
INSERT INTO KMT_DEVICE_TYPE  (UUID, BASE_FAMILY_ID, VERSION, NAME,
ROLE_NAME) VALUES( 1004, 4, 0, 'XIV', 'XIV');
INSERT INTO KMT_DEVICE_TYPE(UUID,BASE_FAMILY_ID,VERSION,NAME,
ROLE_NAME) VALUES(12,12,0,'GPFS','GPFS');




CREATE TABLE KMT_DEVICE_TYPE_ATTRIBUTES (
		ATTR_NAME VARCHAR(256) NOT NULL,
		ATTR_VALUE VARCHAR(2048) NOT NULL,
		DEVICE_TYPE_ID SMALLINT NOT NULL,
		UUID VARCHAR(64) NOT NULL,
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_IDX
	DATA CAPTURE NONE ;

CREATE INDEX DEVTYPE_ATTRNAME_IDX ON KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME ASC, DEVICE_TYPE_ID ASC) PCTFREE 0;

ALTER TABLE KMT_DEVICE_TYPE_ATTRIBUTES ADD CONSTRAINT DEVTYPE_ATTRS_PK PRIMARY KEY (UUID);

ALTER TABLE KMT_DEVICE_TYPE_ATTRIBUTES ADD CONSTRAINT DEVTYPE_FK FOREIGN KEY (DEVICE_TYPE_ID)
	REFERENCES KMT_DEVICE_TYPE (UUID)
	ON DELETE CASCADE;

-- Commented all statements for SYSTEM_X device with devicetypeId = 1003. Add/uncomment later when needed        
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 0, 'DEVTYPEATTR-0');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 1, 'DEVTYPEATTR-1');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 2, 'DEVTYPEATTR-2');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 3, 'DEVTYPEATTR-3');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 4, 'DEVTYPEATTR-4');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('device.enableMachineAffinity', 'false', 4, 'DEVTYPEATTR-12');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 5, 'DEVTYPEATTR-5');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 6, 'DEVTYPEATTR-6');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 7, 'DEVTYPEATTR-7');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 8, 'DEVTYPEATTR-8');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 9, 'DEVTYPEATTR-9');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 10, 'DEVTYPEATTR-10');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 11, 'DEVTYPEATTR-11');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 1000, 'DEVTYPEATTR-1000');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 1001, 'DEVTYPEATTR-1001');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('device.enableMachineAffinity', 'false', 1001, 'DEVTYPEATTR-1002');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 1002, 'DEVTYPEATTR-1003');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('device.enableMachineAffinity', 'false', 1002, 'DEVTYPEATTR-1004');
--INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
--UUID) VALUES ('enableKMIPDelete', 'false', 1003, 'DEVTYPEATTR-1005');
--INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
--UUID) VALUES ('device.enableMachineAffinity', 'false', 1003, 'DEVTYPEATTR-1006');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('enableKMIPDelete', 'false', 1004, 'DEVTYPEATTR-1007');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, DEVICE_TYPE_ID, 
UUID) VALUES ('device.enableMachineAffinity', 'false', 1004, 'DEVTYPEATTR-1008');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('AddNewCertsToPending', 'false', 12, 'DEVTYPEATTR-13');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('enableKMIPDelete', 'false', 12, 'DEVTYPEATTR-14');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('device.AutoPendingAutoDiscovery', '0', 1, 'DEVTYPEATTR-15');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('device.AutoPendingAutoDiscovery', '0', 2, 'DEVTYPEATTR-16');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('device.AutoPendingAutoDiscovery', '0', 3, 'DEVTYPEATTR-17');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('device.AutoPendingAutoDiscovery', '0', 4, 'DEVTYPEATTR-18');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('device.AutoPendingAutoDiscovery', '0', 1000, 'DEVTYPEATTR-19');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('device.AutoPendingAutoDiscovery', '0', 1001, 'DEVTYPEATTR-20');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('device.AutoPendingAutoDiscovery', '0', 1002, 'DEVTYPEATTR-21');
--INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
--UUID) VALUES ('device.AutoPendingAutoDiscovery', '0', 1003, 'DEVTYPEATTR-22');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('device.AutoPendingAutoDiscovery', '0', 1004, 'DEVTYPEATTR-23');
INSERT INTO KMT_DEVICE_TYPE_ATTRIBUTES (ATTR_NAME,ATTR_VALUE,DEVICE_TYPE_ID,
UUID) VALUES ('device.AutoPendingAutoDiscovery', '0', 12, 'DEVTYPEATTR-24');

ALTER TABLE KMT_CERTSTR_RN ADD COLUMN DEVICE_TYPE_ID SMALLINT NOT NULL WITH DEFAULT 0;
ALTER TABLE KMT_CERTSTR_RN ADD COLUMN LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP;

CREATE INDEX KMT_CERTSTR_RN_DEVTYPE_IDX ON KMT_CERTSTR_RN (UUID ASC, DEVICE_TYPE_ID ASC) PCTFREE 0;

ALTER TABLE KMT_CERTSTR_RN ADD CONSTRAINT CERT_DEVTYPE_FK FOREIGN KEY (DEVICE_TYPE_ID)
	REFERENCES KMT_DEVICE_TYPE (UUID)
	ON DELETE CASCADE;


ALTER TABLE KMT_KEYSTR_RN ADD COLUMN DEVICE_TYPE_ID SMALLINT NOT NULL WITH DEFAULT 0;
ALTER TABLE KMT_KEYSTR_RN ADD COLUMN LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP;


CREATE INDEX KMT_KEYSTR_RN_DEVTYPE_IDX ON KMT_KEYSTR_RN (UUID ASC, DEVICE_TYPE_ID ASC) PCTFREE 0;

ALTER TABLE KMT_KEYSTR_RN ADD CONSTRAINT KEY_DEVTYPE_FK FOREIGN KEY (DEVICE_TYPE_ID)
	REFERENCES KMT_DEVICE_TYPE (UUID)
	ON DELETE CASCADE;

	
ALTER TABLE KMT_GROUP ADD COLUMN DEVICE_TYPE_ID SMALLINT NOT NULL WITH DEFAULT 0 ;
ALTER TABLE KMT_GROUP ADD COLUMN LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP;


CREATE INDEX KMT_GROUP_DEVTYPE_IDX ON KMT_GROUP (UUID ASC, DEVICE_TYPE_ID ASC) PCTFREE 0;

ALTER TABLE KMT_GROUP ADD CONSTRAINT GROUP_DEVTYPE_FK FOREIGN KEY (DEVICE_TYPE_ID)
	REFERENCES KMT_DEVICE_TYPE (UUID)
	ON DELETE CASCADE;
ALTER TABLE KMT_DEVAUDIT ADD COLUMN DEVICE_TYPE_NAME VARCHAR(64) WITH DEFAULT 'UNSET';
ALTER TABLE KMT_DEVAUDIT ADD COLUMN LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP;

CREATE INDEX KMT_DEVAUDIT_DEVTYPENAME_IDX ON KMT_DEVAUDIT (DEVICE_TYPE_NAME) PCTFREE 0;

------------------------------- END DEVICE TYPE RELATED UPDATES --------------------------------

------------------------------- START MACHINE AFFINITY UPDATES --------------------------------

CREATE TABLE KMT_MACHINE (
    UUID VARCHAR(64) NOT NULL,
    MACHINEID VARCHAR(64) NOT NULL,
    DESCRIPTION VARCHAR(256),
    MACHINETEXT VARCHAR (256),
	FAMILYTYPE SMALLINT,
	VERSION INTEGER NOT NULL,
	LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP	
  )
  IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
  DATA CAPTURE NONE ;
 
ALTER TABLE KMT_MACHINE ADD CONSTRAINT PK_MACHINE_ID PRIMARY KEY (UUID);
CREATE UNIQUE INDEX IDX_MACHINE_01 On KMT_MACHINE(MACHINEID, MACHINETEXT) MINPCTUSED 20;

CREATE TABLE KMT_MACHINE_DEVICE (
    DEVICEID VARCHAR(64) NOT NULL,
    MACHINEID  VARCHAR(64) NOT NULL,
    PENDING_DATE TIMESTAMP,
	PENDINGAUTO SMALLINT,
	VERSION INTEGER NOT NULL,
	LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP	
  )	
  IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
  DATA CAPTURE NONE ;
 
ALTER TABLE KMT_MACHINE_DEVICE ADD CONSTRAINT PK_MACHE_DEV_ID PRIMARY KEY (DEVICEID, MACHINEID);
ALTER TABLE KMT_MACHINE_DEVICE ADD CONSTRAINT MACHINE_DEVICE_FK1 FOREIGN KEY (DEVICEID)
   REFERENCES KMT_DEVICE (UUID);
ALTER TABLE KMT_MACHINE_DEVICE ADD CONSTRAINT MACHINE_DEVICE_FK2 FOREIGN KEY (MACHINEID)
   REFERENCES KMT_MACHINE (UUID);

------------------------------- END MACHINE AFFINITY UPDATES --------------------------------

------------------------------- START KMIP RELATED UPDATES --------------------------------
CREATE TABLE KMT_KMIP_ATTR_APPINFO (
		MANAGED_OBJECT_UUID VARCHAR(64) NOT NULL,
		APP_NAMESPACE VARCHAR(1024) NOT NULL,
		APP_INFO VARCHAR(2048) NOT NULL,
		INDEX_ID SMALLINT NOT NULL,
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_ATTR_APPINFO ADD CONSTRAINT KMT_APPINFO_PK PRIMARY KEY (MANAGED_OBJECT_UUID, INDEX_ID);
CREATE INDEX KMT_KMIP_ATTR_APPINFO_IDX ON KMT_KMIP_ATTR_APPINFO (MANAGED_OBJECT_UUID) PCTFREE 0;

CREATE TABLE KMT_KMIP_ATTR_CERTSUBJECT (
		CERT_UUID VARCHAR(64) NOT NULL,
		ALTERNATIVE_NAME VARCHAR(1024),
		INDEX_ID SMALLINT NOT NULL,
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_ATTR_CERTSUBJECT ADD CONSTRAINT FK_CERT FOREIGN KEY (CERT_UUID)
	REFERENCES KMT_CERT (UUID)
	ON DELETE CASCADE;
ALTER TABLE KMT_KMIP_ATTR_CERTSUBJECT ADD CONSTRAINT KMT_CERTSUB_PK PRIMARY KEY (CERT_UUID, INDEX_ID);

CREATE TABLE KMT_KMIP_ATTR_CERTISSUER (
		CERT_UUID VARCHAR(64) NOT NULL,
		ALTERNATIVE_NAME VARCHAR(1024),
		INDEX_ID SMALLINT NOT NULL,
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_ATTR_CERTISSUER ADD CONSTRAINT FK_CERT FOREIGN KEY (CERT_UUID)
	REFERENCES KMT_CERT (UUID)
	ON DELETE CASCADE;
ALTER TABLE KMT_KMIP_ATTR_CERTISSUER ADD CONSTRAINT KMT_CERTISS_PK PRIMARY KEY (CERT_UUID, INDEX_ID);

CREATE TABLE KMT_KMIP_ATTR_CRYPTOPARAMS (
		BLOCK_CIPHER_MODE INTEGER,
		PADDING_METHOD INTEGER,
		HASHING_ALGORITHM INTEGER,
		ROLE_TYPE INTEGER,
		MANAGED_OBJECT_UUID VARCHAR(64) NOT NULL,
		INDEX_ID SMALLINT NOT NULL,
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_ATTR_CRYPTOPARAMS ADD CONSTRAINT PK_CRYPTOPARAMS PRIMARY KEY (MANAGED_OBJECT_UUID, INDEX_ID);
CREATE INDEX KMT_KMIP_ATTR_CRYPTOPARAMS_IDX ON KMT_KMIP_ATTR_CRYPTOPARAMS (MANAGED_OBJECT_UUID) PCTFREE 0;

CREATE TABLE KMT_KMIP_ATTR_CUSTOM (
		MANAGED_OBJECT_UUID VARCHAR(64) NOT NULL,
		TYPE INTEGER,
		ATTR_NAME VARCHAR(1024) NOT NULL,
		ATTR_VALUE VARCHAR(2048) NOT NULL,
		INDEX_ID SMALLINT NOT NULL,
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_ATTR_CUSTOM ADD CONSTRAINT PK_CUSTOMATTR PRIMARY KEY (MANAGED_OBJECT_UUID, ATTR_NAME, INDEX_ID);
CREATE INDEX KMT_KMIP_ATTR_CUSTOM_IDX ON KMT_KMIP_ATTR_CUSTOM (MANAGED_OBJECT_UUID) PCTFREE 0;

CREATE TABLE KMT_KMIP_ATTR_DIGEST (
		HASHING_ALGORITHM INTEGER NOT NULL,
		DIGEST_VALUE VARCHAR(1024) NOT NULL,
		MANAGED_OBJECT_UUID VARCHAR(64) NOT NULL,
		INDEX_ID SMALLINT NOT NULL,
		KEY_FORMAT SMALLINT NOT NULL,
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_ATTR_DIGEST ADD CONSTRAINT PK_DIGEST PRIMARY KEY (MANAGED_OBJECT_UUID, INDEX_ID);
CREATE INDEX KMT_KMIP_ATTR_DIGEST_IDX ON KMT_KMIP_ATTR_DIGEST (MANAGED_OBJECT_UUID) PCTFREE 0;

CREATE TABLE KMT_KMIP_ATTR_LINK (
		LINK_TARGET_OBJECT_UUID VARCHAR(64) NOT NULL,
		LINK_TYPE INTEGER NOT NULL,
		LINK_SOURCE_OBJECT_UUID VARCHAR(64) NOT NULL,
		INDEX_ID SMALLINT NOT NULL,
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_ATTR_LINK ADD CONSTRAINT PK_LINK PRIMARY KEY (LINK_SOURCE_OBJECT_UUID, INDEX_ID);
CREATE INDEX KMT_KMIP_ATTR_LINK_IDX1 ON KMT_KMIP_ATTR_LINK (LINK_TARGET_OBJECT_UUID) PCTFREE 0;
CREATE INDEX KMT_KMIP_ATTR_LINK_IDX2 ON KMT_KMIP_ATTR_LINK (LINK_SOURCE_OBJECT_UUID) PCTFREE 0;

CREATE TABLE KMT_KMIP_GLOBAL_NAMES (                
		NAME_TYPE INTEGER NOT NULL,
		NAME_VALUE VARCHAR(512) NOT NULL,
		MANAGED_OBJECT_UUID VARCHAR(64) NOT NULL,
        NAME_UUID VARCHAR(64) NOT NULL,
        LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
  )
  IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
  DATA CAPTURE NONE;
ALTER TABLE KMT_KMIP_GLOBAL_NAMES ADD CONSTRAINT KMT_KMIP_GN_PK PRIMARY KEY (NAME_UUID);
ALTER TABLE KMT_KMIP_GLOBAL_NAMES ADD CONSTRAINT UNQ_NAME UNIQUE (NAME_VALUE, NAME_TYPE);
CREATE INDEX KMT_KMIP_GLOBAL_NAMES_IDX ON KMT_KMIP_GLOBAL_NAMES (MANAGED_OBJECT_UUID) PCTFREE 0;

CREATE TABLE KMT_KMIP_ATTR_NAME (                
		MANAGED_OBJECT_UUID VARCHAR(64) NOT NULL,
		INDEX_ID SMALLINT NOT NULL,
        NAME_UUID VARCHAR(64) NOT NULL,
        LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
  )
  IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
  DATA CAPTURE NONE;
ALTER TABLE KMT_KMIP_ATTR_NAME 
    ADD CONSTRAINT UNQ_INSTANCE 
    UNIQUE (MANAGED_OBJECT_UUID, INDEX_ID);
ALTER TABLE KMT_KMIP_ATTR_NAME 
    ADD CONSTRAINT FK 
    FOREIGN KEY (NAME_UUID) REFERENCES KMT_KMIP_GLOBAL_NAMES (NAME_UUID) ON DELETE CASCADE;
CREATE INDEX KMT_KMIP_ATTR_NAME_IDX ON KMT_KMIP_ATTR_NAME (MANAGED_OBJECT_UUID) PCTFREE 0;
CREATE INDEX KMT_KMIP_ATTR_NAME_IDX2 ON KMT_KMIP_ATTR_NAME (NAME_UUID) PCTFREE 0;

CREATE TABLE KMT_KMIP_ATTR_OBJECTGROUP (
		GROUP_NAME VARCHAR(512) NOT NULL,
		MEMBER_OBJECT_UUID VARCHAR(64) NOT NULL,
		INDEX_ID SMALLINT NOT NULL,
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_ATTR_OBJECTGROUP ADD CONSTRAINT PK_OBJGROUP PRIMARY KEY (MEMBER_OBJECT_UUID, INDEX_ID);
CREATE INDEX KMT_KMIP_ATTR_OBJECTGROUP_IDX ON KMT_KMIP_ATTR_OBJECTGROUP (MEMBER_OBJECT_UUID) PCTFREE 0;

-- KMIP 1.2 addition
CREATE TABLE KMT_KMIP_ATTR_ALTERNATIVE_NAME (
		MANAGED_OBJECT_UUID VARCHAR(64) NOT NULL,
		ALTERNATIVE_NAME VARCHAR(512) NOT NULL,
		TYPE SMALLINT NOT NULL,
		INDEX_ID SMALLINT NOT NULL,
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_4K_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_ATTR_ALTERNATIVE_NAME ADD CONSTRAINT PK_ALTNAME PRIMARY KEY (MANAGED_OBJECT_UUID, INDEX_ID);
CREATE INDEX KMT_KMIP_ATTR_ALTERNATIVE_NAME_IDX ON KMT_KMIP_ATTR_ALTERNATIVE_NAME (MANAGED_OBJECT_UUID) PCTFREE 0;
CREATE INDEX KMT_KMIP_ATTR_ALTERNATIVE_NAME_IDX2 ON KMT_KMIP_ATTR_ALTERNATIVE_NAME (ALTERNATIVE_NAME) PCTFREE 0;
CREATE INDEX KMT_KMIP_ATTR_ALTERNATIVE_NAME_IDX3 ON KMT_KMIP_ATTR_ALTERNATIVE_NAME (TYPE) PCTFREE 0;
-- end of KMIP 1.2 addition

-- outplan ?
CREATE TABLE KMT_KMIP_OBJECT_OPAQUE (
		UUID VARCHAR(64) NOT NULL,
		ARCHIVE_DATE TIMESTAMP,
		LAST_CHANGED_DATE TIMESTAMP,
		INITIAL_DATE TIMESTAMP NOT NULL,
		DESTROYDATE TIMESTAMP,
		ACTIVATIONDATE TIMESTAMP,
		DATA VARCHAR (16000) FOR BIT DATA,
		LARGE_DATA BLOB,
		CONTACT_INFO VARCHAR(1024),
		VERSION INT NOT NULL,	
		OPERATIONAL_POLICY VARCHAR(64),
		USAGE_VALUE DECIMAL(18,0),
		USAGE_TOTAL DECIMAL(18,0),
		USAGE_LIMIT_TYPE SMALLINT,
		OWNER VARCHAR(128),
		ORIGINAL_CREATION_DATE TIMESTAMP
	)
	IN TKLM_DATA_32K_LG INDEX IN TKLM_32K_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_OBJECT_OPAQUE ADD CONSTRAINT PK_OBJ_OPAQUE PRIMARY KEY (UUID);
--ALTER TABLE KMT_KMIP_OBJECT_OPAQUE ADD CONSTRAINT FK_OP_POLICY FOREIGN KEY (OPERATIONAL_POLICY)
--	REFERENCES KMT_KMIP_ATTR_OPERATIONAL_POLICY (UUID);
CREATE INDEX IDX_OPQOBJ_01 ON KMT_KMIP_OBJECT_OPAQUE(INITIAL_DATE) MINPCTUSED 20;
CREATE INDEX IDX_OPQOBJ_02 ON KMT_KMIP_OBJECT_OPAQUE (ACTIVATIONDATE) MINPCTUSED 20;
CREATE INDEX IDX_OPQOBJ_03 ON KMT_KMIP_OBJECT_OPAQUE (DESTROYDATE) MINPCTUSED 20;
CREATE INDEX IDX_OPQOBJ_04 ON KMT_KMIP_OBJECT_OPAQUE (OWNER) MINPCTUSED 20;
CREATE INDEX IDX_OPQOBJ_05 ON KMT_KMIP_OBJECT_OPAQUE (ORIGINAL_CREATION_DATE) MINPCTUSED 20;

CREATE TABLE KMT_KMIP_OBJECT_SECRET_DATA (
		UUID VARCHAR(64) NOT NULL,
		TYPE INT NOT NULL,
		CRYPTO_LENGTH INTEGER,			
		ACTIVATIONDATE TIMESTAMP,
		DEACTIVATION_DATE TIMESTAMP,		
		ARCHIVE_DATE TIMESTAMP,
		INITIAL_DATE TIMESTAMP NOT NULL,
		LAST_CHANGED_DATE TIMESTAMP NOT NULL,
		DESTROYDATE TIMESTAMP,
		COMPROMISE_OCCURRENCE_DATE TIMESTAMP,
		COMPROMISEDATE TIMESTAMP,
		REVOKE_REASON_CODE INTEGER,
		REVOKE_REASON_MESSAGE VARCHAR(2048),
		CONTACT_INFO VARCHAR(1024),
		ALIAS VARCHAR(256) NOT NULL, 
		KEYPASSWORD VARCHAR(256),
		KEYSTORE_UUID VARCHAR(64) NOT NULL,
		VERSION INT NOT NULL,	
		OPERATIONAL_POLICY VARCHAR(64),
		OWNER VARCHAR(128),
		LEASE_TIME INTEGER,
		PREVSTATE INTEGER,
		FORMAT SMALLINT,
		CRYPTO_USAGE VARCHAR(32),
		ORIGINAL_CREATION_DATE TIMESTAMP
	)
	IN TKLM_DATA_LG INDEX IN TKLM_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_OBJECT_SECRET_DATA ADD CONSTRAINT PK_SECDATA PRIMARY KEY (UUID);
ALTER TABLE KMT_KMIP_OBJECT_SECRET_DATA ADD CONSTRAINT FK_KS_UUID FOREIGN KEY (KEYSTORE_UUID)
	REFERENCES KMT_KEYSTR (UUID) ON DELETE RESTRICT;
ALTER TABLE KMT_KMIP_OBJECT_SECRET_DATA ADD CONSTRAINT UN_SECDATA_1 UNIQUE (KEYSTORE_UUID,ALIAS);
CREATE INDEX IDX_SECDATA_01 ON KMT_KMIP_OBJECT_SECRET_DATA (INITIAL_DATE) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_02 ON KMT_KMIP_OBJECT_SECRET_DATA (DEACTIVATION_DATE) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_03 ON KMT_KMIP_OBJECT_SECRET_DATA (ACTIVATIONDATE) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_04 ON KMT_KMIP_OBJECT_SECRET_DATA (COMPROMISEDATE) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_05 ON KMT_KMIP_OBJECT_SECRET_DATA (DESTROYDATE) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_06 ON KMT_KMIP_OBJECT_SECRET_DATA (LEASE_TIME) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_07 ON KMT_KMIP_OBJECT_SECRET_DATA (OWNER) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_08 ON KMT_KMIP_OBJECT_SECRET_DATA (REVOKE_REASON_CODE) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_09 ON KMT_KMIP_OBJECT_SECRET_DATA (ALIAS) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_10 ON KMT_KMIP_OBJECT_SECRET_DATA (PREVSTATE) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_11 ON KMT_KMIP_OBJECT_SECRET_DATA (CRYPTO_USAGE) MINPCTUSED 20;
CREATE INDEX IDX_SECDATA_12 ON KMT_KMIP_OBJECT_SECRET_DATA (ORIGINAL_CREATION_DATE) MINPCTUSED 20;

CREATE TABLE KMT_KMIP_OBJECT_TEMPLATE (
		UUID VARCHAR(64) NOT NULL,
		ALGORITHM VARCHAR(512),
		CRYPTO_LENGTH INTEGER,
		CONTACT_INFO VARCHAR(1024),
		INITIAL_DATE TIMESTAMP NOT NULL,
		ACTIVATIONDATE TIMESTAMP,
		DESTROYDATE TIMESTAMP,
		ARCHIVE_DATE TIMESTAMP,
		LAST_CHANGED_DATE TIMESTAMP NOT NULL,
		CRYPTO_USAGE VARCHAR(32),
		PROCESS_START_DATE TIMESTAMP,
		PROTECT_STOP_DATE TIMESTAMP,
		DEACTIVATION_DATE TIMESTAMP,
		VERSION INT NOT NULL,
		USAGE_VALUE DECIMAL(18,0),
		USAGE_TOTAL DECIMAL(18,0),
		USAGE_LIMIT_TYPE SMALLINT,		
		OWNER VARCHAR(128),
		OPERATIONAL_POLICY VARCHAR(64),
-- new field                
		BODY_UUID VARCHAR(64),
		ORIGINAL_CREATION_DATE TIMESTAMP
	)
	IN TKLM_DATA_LG INDEX IN TKLM_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_OBJECT_TEMPLATE ADD CONSTRAINT PK_TEMPLATE PRIMARY KEY (UUID);
CREATE INDEX IDX_TMPL_01 ON KMT_KMIP_OBJECT_TEMPLATE (INITIAL_DATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_02 ON KMT_KMIP_OBJECT_TEMPLATE (DEACTIVATION_DATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_03 ON KMT_KMIP_OBJECT_TEMPLATE (ACTIVATIONDATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_04 ON KMT_KMIP_OBJECT_TEMPLATE (DESTROYDATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_05 ON KMT_KMIP_OBJECT_TEMPLATE (ALGORITHM) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_06 ON KMT_KMIP_OBJECT_TEMPLATE (CRYPTO_LENGTH) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_07 ON KMT_KMIP_OBJECT_TEMPLATE (OWNER) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_08 ON KMT_KMIP_OBJECT_TEMPLATE (PROCESS_START_DATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_09 ON KMT_KMIP_OBJECT_TEMPLATE (PROTECT_STOP_DATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_10 ON KMT_KMIP_OBJECT_TEMPLATE (USAGE_VALUE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_11 ON KMT_KMIP_OBJECT_TEMPLATE (USAGE_TOTAL) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_12 ON KMT_KMIP_OBJECT_TEMPLATE (USAGE_LIMIT_TYPE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_13 ON KMT_KMIP_OBJECT_TEMPLATE (CRYPTO_USAGE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_14 ON KMT_KMIP_OBJECT_TEMPLATE (BODY_UUID) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_15 ON KMT_KMIP_OBJECT_TEMPLATE (ORIGINAL_CREATION_DATE) MINPCTUSED 20;

CREATE TABLE KMT_KMIP_OBJECT_TEMPLATE_BODY (
		UUID VARCHAR(64) NOT NULL,
		PARENT_UUID VARCHAR(64) NOT NULL,
		ALGORITHM VARCHAR(512),
		CRYPTO_LENGTH INTEGER,
		CONTACT_INFO VARCHAR(1024),
		INITIAL_DATE TIMESTAMP NOT NULL,
		ACTIVATIONDATE TIMESTAMP,
		DESTROYDATE TIMESTAMP,
		ARCHIVE_DATE TIMESTAMP,
		LAST_CHANGED_DATE TIMESTAMP NOT NULL,
		CRYPTO_USAGE VARCHAR(32),
		PROCESS_START_DATE TIMESTAMP,
		PROTECT_STOP_DATE TIMESTAMP,
		DEACTIVATION_DATE TIMESTAMP,
		VERSION INT NOT NULL,
		USAGE_VALUE DECIMAL(18,0),
		USAGE_TOTAL DECIMAL(18,0),
		USAGE_LIMIT_TYPE SMALLINT,		
		OWNER VARCHAR(128),
		OPERATIONAL_POLICY VARCHAR(64)
	)
	IN TKLM_DATA_LG INDEX IN TKLM_IDX
	DATA CAPTURE NONE ;
ALTER TABLE KMT_KMIP_OBJECT_TEMPLATE_BODY ADD CONSTRAINT PK_TEMPLATE_BODY PRIMARY KEY (UUID);
CREATE INDEX IDX_TMPL_B_01 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (INITIAL_DATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_02 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (DEACTIVATION_DATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_03 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (ACTIVATIONDATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_04 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (DESTROYDATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_05 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (ALGORITHM) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_06 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (CRYPTO_LENGTH) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_07 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (OWNER) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_08 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (PROCESS_START_DATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_09 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (PROTECT_STOP_DATE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_10 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (USAGE_VALUE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_11 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (USAGE_TOTAL) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_12 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (USAGE_LIMIT_TYPE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_13 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (CRYPTO_USAGE) MINPCTUSED 20;
CREATE INDEX IDX_TMPL_B_14 ON KMT_KMIP_OBJECT_TEMPLATE_BODY (PARENT_UUID) MINPCTUSED 20;

ALTER TABLE KMT_KMIP_OBJECT_TEMPLATE_BODY 
    ADD CONSTRAINT FK 
    FOREIGN KEY (PARENT_UUID) REFERENCES KMT_KMIP_OBJECT_TEMPLATE (UUID) ON DELETE CASCADE;

CREATE TABLE KMT_CLIENT (
		IP_HOSTNAME VARCHAR(255),
		PORT INTEGER,
		PRI_EMAIL VARCHAR(256),
		PRI_PHONE VARCHAR(15),
		BKUP_EMAIL VARCHAR(256),
		BKUP_PHONE VARCHAR(15),
		CAPABILITIES VARCHAR(512),
		MECHANISMS VARCHAR(256),
		INFO VARCHAR(256),
		CRYPTO_USAGE VARCHAR(32),
		UUID VARCHAR(128) PRIMARY KEY NOT NULL,
		NAME VARCHAR(64),
		TYPE INTEGER NOT NULL,
		DEV_TYPE_UUID SMALLINT,
		ADD_COL_1 VARCHAR(256),
		ADD_COL_2 VARCHAR(256),
		ADD_COL_3 VARCHAR(256),
		ADD_COL_4 VARCHAR(256),
		ADD_COL_5 VARCHAR(256),
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP,
		CONSTRAINT CL_FK1 FOREIGN KEY (DEV_TYPE_UUID) REFERENCES KMT_DEVICE_TYPE (UUID) ON DELETE CASCADE
	);


CREATE TABLE KMT_CLIENT_KEY_CERT_LIST (
		CLIENT_UUID VARCHAR(128) NOT NULL,
		KEYCERT_UUID VARCHAR(128) NOT NULL,
		KEYCERT_HASH VARCHAR(256) FOR BIT DATA,
		ADD_COL_1 VARCHAR(256),
		ADD_COL_2 VARCHAR(256),
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	);

ALTER TABLE KMT_CLIENT_KEY_CERT_LIST ADD CONSTRAINT CKL_FK1 FOREIGN KEY (CLIENT_UUID)
  REFERENCES KMT_CLIENT (UUID)
  ON DELETE CASCADE;
  
ALTER TABLE KMT_CLIENT_KEY_CERT_LIST ADD CONSTRAINT CKL_FK2 FOREIGN KEY (KEYCERT_UUID)
  REFERENCES KMT_CERT (UUID)
  ON DELETE CASCADE;
  
CREATE INDEX IDX_CLT_KL_01 ON KMT_CLIENT_KEY_CERT_LIST (CLIENT_UUID, KEYCERT_UUID) MINPCTUSED 20;
CREATE INDEX IDX_CLT_KL_02 ON KMT_CLIENT_KEY_CERT_LIST (CLIENT_UUID) MINPCTUSED 20;
CREATE INDEX IDX_CLT_KL_03 ON KMT_CLIENT_KEY_CERT_LIST (KEYCERT_UUID) MINPCTUSED 20;


CREATE TABLE KMT_NOTIFICATIONS (
		UUID VARCHAR(128) NOT NULL,
		CLIENT_UUID VARCHAR(128) NOT NULL,
		KEYCERT_UUID VARCHAR(128) NOT NULL,
		STATE INTEGER,
		NOTIFY_POLICY_UUID VARCHAR(128) NOT NULL,
		NUM_NOTIFY_ATTEMPTS INTEGER NOT NULL,
		NUM_NOTIFY_REMAINING INTEGER NOT NULL,
		TIMESTAMP_LAST_NOTIFY TIMESTAMP,
		ADD_COL_1 VARCHAR(256),
		ADD_COL_2 VARCHAR(256),
		ADD_COL_3 VARCHAR(256),
		ADD_COL_4 VARCHAR(256),
		ADD_COL_5 VARCHAR(256),
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	);

ALTER TABLE KMT_NOTIFICATIONS ADD CONSTRAINT NTFN_FK1 FOREIGN KEY (CLIENT_UUID)
  REFERENCES KMT_CLIENT (UUID)
  ON DELETE CASCADE;
  
ALTER TABLE KMT_NOTIFICATIONS ADD CONSTRAINT NTFN_FK2 FOREIGN KEY (KEYCERT_UUID)
  REFERENCES KMT_KEY (UUID)
  ON DELETE CASCADE;
  
ALTER TABLE KMT_NOTIFICATIONS ADD CONSTRAINT NTFN_FK3 FOREIGN KEY (KEYCERT_UUID)
  REFERENCES KMT_CERT (UUID)
  ON DELETE CASCADE;

CREATE INDEX IDX_NOTIFN_01 ON KMT_NOTIFICATIONS (UUID) MINPCTUSED 20;

CREATE TABLE KMT_NOTIFICATION_POLICY (
		UUID VARCHAR(128) NOT NULL,
		LONG_NOTIFY_PERIOD INTEGER,
		SHORT_NOTIFY_PERIOD INTEGER,
		PRI_NOTIFY_TYPE INTEGER,
		BKUP_NOTIFY_TYPE INTEGER,
		NUM_NOTIFY_FAILURES INTEGER,
		NUM_NOTIFY INTEGER,
		ADD_COL_1 VARCHAR(256),
		ADD_COL_2 VARCHAR(256),
		ADD_COL_3 VARCHAR(256),
		ADD_COL_4 VARCHAR(256),
		ADD_COL_5 VARCHAR(256),
		LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	);

CREATE INDEX IDX_NOTIFN_PLCY_01 ON KMT_NOTIFICATION_POLICY (UUID) MINPCTUSED 20;
		


------------------------------- END KMIP RELATED UPDATES ----------------------------------

-------------------------------- START DELL LICENSE UPDATES --------------------------------

CREATE TABLE KMT_LICENSE (
    DEVICETYPE SMALLINT NOT NULL,
    COUNT INTEGER NOT NULL, 
    CREATIONTIME TIMESTAMP NOT NULL,
    LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
    )
    IN TKLM_DATA_SM INDEX IN TKLM_IDX
    DATA CAPTURE NONE;

ALTER TABLE KMT_LICENSE ADD CONSTRAINT PK_LICENSE PRIMARY KEY (DEVICETYPE);

------------------------------- END DELL LICENSE UPDATES ----------------------------------
-------------------------------- START DELL BACKUP BEFORE SERVING UPDATES --------------------------------

CREATE TABLE KMT_BACKUP (
	LASTBACKUPDATE TIMESTAMP,
	LAST_UPDATE_TIMESTAMP TIMESTAMP NOT NULL GENERATED BY DEFAULT FOR EACH ROW ON UPDATE AS ROW CHANGE TIMESTAMP
	)
	IN TKLM_DATA_SM INDEX IN TKLM_IDX
    DATA CAPTURE NONE;

INSERT INTO KMT_BACKUP (LASTBACKUPDATE) VALUES (NULL);
	
------------------------------- END DELL BACKUP BEFORE SERVING UPDATES ----------------------------------
------------------------------- END V2 UPDATES -------------------------------------------------

-- commits the changes to DB2
commit;

