#!/bin/bash

#########################################################
#
#       EDIT THE PARAMETER VALUE IN THIS SECTION
#
# tiphome : Required, the home directory of Tivoli Integrated Portal
# username : Required, the username of the Tivoli Key Lifecycle Manager user with klmBackup permission
# password : Required, the password for the Tivoli Key Lifecycle Manager user to login
# backuppw : Required, the password used for backup operation
# backupdes : Optional, description of the Tivoli Key Lifecycle Manager backup
# backupdir : Optional, full path to the directory, where the backup jar file is to be stored
# backupDBdir : Optional, full path to the directory, where the database backup is to be stored
tiphome=
username=
password=
backuppw=
backupdes=
backupdir=
backupDBdir=
###########################################################


###########################################################
# 
#       DO NOT MAKE CHANGES BELOW THIS LINE
#   
###########################################################
backupresult=
backupparams=
bashpath=$(dirname $0)
if [ ! "$tiphome" ]
then
  echo Tivoli Integrated Portal home directory is not specified.
  backupresult=104
elif [ ! "$username" ]
then
  echo Tivoli Key Lifecycle Manager username is not specified.
  backupresult=104
elif [ ! "$password" ]
then
  echo Tivoli Key Lifecycle Manager password is not specified.
  backupresult=104
elif [ ! "$backuppw" ]
then
  echo Backup password is not specified.
  backupresult=104
else
  if [ "$backupdes" ]
  then
    backupparams="$backupparams -description \"$backupdes\""
  fi
  if [ "$backupdir" ]
  then
    backupparams="$backupparams -backupDirectory \"$backupdir\""
  fi
  if [ "$backupDBdir" ]
  then
    backupparams="$backupparams -databaseBackupDirectory \"$backupDBdir\""
  fi

  $tiphome/bin/wsadmin.sh -username "$username" -password "$password" -lang jython -f $bashpath/autobackup.py "$backuppw" $backupparams
  backupresult=$?
  
  case $backupresult in
  0 )
    echo Backup is successful.
    ;;
  2 )
    echo Backup is partially successful.
    ;;
  3 )
    echo Backup is not needed.
    ;;
  4 )
    echo Backup is running. No backup is created by this script.
    ;;
  101 )
    echo Backup is unsuccessful.
    ;;
  102 )
    echo Backup result is unknown.
    ;;
  103 )
    echo "Exception in auto backup process. No backup is created."
    ;;
  105 )
    echo Login to Tivoli Key Lifecycle Manager failed.
    ;;
  * )
    echo Tivoli Integrated Portal home directory is incorrect.
    ;;
  esac
fi

echo Auto backup result status : $backupresult

#Auto backup result status
#0 : backup is successful
#2 : backup is partially successful
#3 : backup is not needed
#4 : backup is running
#101 : backup is unsuccessful
#102 : backup result is unknown
#103 : exception during auto backup process
#104 : script error, missing required parameter etc
#105 : login failed
# other number (127) : Tivoli Integrated Portal home directory is incorrect.