#!/bin/ksh
#set -x
#
# Licensed Materials-Property of IBM
# (C)Copyright IBM Corp 2009
# All rights reserved
# US Government Users Restricted Rights
# Use, duplication, or disclosure restricted
# by GSA ADP Schedule Contract with IBM Corp
# 
# The script is to be called from the IBM Tivoli Key Life Cycle Manager 
# Migration tool to migrate the database schema from V1 to V2
#
# arg1 -  log  -   log file to be used by this script 
# arg2 -  instance  -  instance name (for example db2inst2) 
# arg3 -  dbname  -  database name
# arg4 -  dbuserid  - database instance owner
# arg5 -  dbpassword  - database password
# arg6 -  dbinstance_owner_home - (For example /home/db2inst2)
#
#  Note:  The script 
#	        must be run on the same machine where DB2 server is installed.
#		does do not parameter validation
#	  	will fail if the user specifies a wrong dbuserid or password
#	        uses the dbusername for the db2fence ID
#		uses the database name as it alias
#
#------------------------------------------------------------------------------

LOG=$1
DB_INST=$2
DB_NAME=$3
DB_USERID=$4
DB_PASSWORD=$5
INST_HOME=$6

#flag to indicate if we are connected to the database
is_connected=0
#The return code from the script
rc=0

if [ $# -ne 6 ]
then
echo Usage: dbtklmupd.sh logfile dbinstance dbname dbusername dbpassword dbinstance_owner_home
    exit 2
fi


##################################################################
echo "--------------------------dbtklmupd.sh----------------------------------------------------------------------" >> $LOG

CURRENT_DIR=`dirname $0`
if [ "$CURRENT_DIR" = "." ] ; then
   CURRENT_DIR=`pwd`
fi

ret=0

# set instance profile path to source profile of local instance

#source $DB_INST/db2profile
echo . $INST_HOME/sqllib/db2profile >> "$LOG" 
. $INST_HOME/sqllib/db2profile
rc=$?
   
if [ $rc -eq 0 ]; then
   echo db2 start database manager >> "$LOG"
   db2 -l "$LOG" "start database manager"
   #Just assume database manager started
fi

#Connect to the database specifying the id and the password
if [ $rc -eq 0 ]; then 
    echo "db2 connect to $DB_NAME user $DB_USERID using ****" >> "$LOG"
    db2 -l "$LOG" "connect to $DB_NAME user $DB_USERID using '$DB_PASSWORD'"
    rc=$?
   #ignore warningings
   if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
      rc=0
   fi
fi

#We are connected to the database
if [ $rc -eq 0 ] 
then
   is_connected=1
   else
   is_connected=0
   rc=1
fi

#Create database objects added in V2
if [ $rc -eq 0 ] 
then 
   echo db2 -s -t -f $CURRENT_DIR/tklmupdsql.db2 >> "$LOG"
   db2 -l "$LOG" -s -t -f "$CURRENT_DIR/tklmupdsql.db2"
   rc=$?
   #ignore warningings
   if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
      rc=0
   fi
fi




#If an error has occurred then perform a rollback
if [ $rc -gt 0 ] ; then
    echo db2 rollback >> "$LOG"
    db2 rollback >> "$LOG"
fi

#Disconnect from the database, no need to check for return code
if [ $is_connected -eq 1 ] ; then
   echo db2 connect reset >> "$LOG"
   db2 -l "$LOG" "connect reset"
   is_connected=0
fi

echo "---------------------------------------------------------------------------------------------------" >> $LOG
if [ $rc -eq 0 ]; then
echo "Return=$rc"
echo "Return=$rc" >> $LOG
else
echo "Return $rc"
echo "Return $rc" >> $LOG
fi
echo "---------------------------------------------------------------------------------------------------" >> $LOG
exit $rc

