#!/bin/ksh
#set -x
#
#
# File Name: getDiskSpace.sh
#
# IBM Confidential
# Object Code Only Source Materials
# 
# (c) Copyright International Business Machines Corp. 1994, 2005
# The source code for this program is not published or otherwise divested
# of its trade secrets, irrespective of what has been deposited with the
# U.S. Copyright Office.
#
#
# This script will run the command passed on the command line and write the
# output to a temp file.  The reason this is necessary is because various 
# commands do not always return control back to the caller because of a battle 
# over i/o.  It also
# sets up library paths for java's sake.  After execution, the outputs will be
# concatenated to stdout and can be captured at that time.
#


#
#  This script will be used to determine how much free
#  disk space is on a volume, or the volume name that 
#  holds a particular path, or both.
#


checkUsage() {
    USAGE="`basename $0` [-f | -s | -b] <path>"
    DIR=`dirname $0`
    DIR=`dirname $DIR` 
    export DIR
    LOG=$DIR/migrate_ds.log 
    export LOG
    rm -f $LOG
    if [ $# -ne 1 -a $# -ne 2 ] 
    then
        echo ${USAGE}
        echo Return 1
        exit 1
    fi

    while getopts f:s:b: flag
    do
        case $flag in
            b) theType="-b"; thePath=$OPTARG ;;
            s) theType="-s"; thePath=$OPTARG ;;
            f) theType="-f"; thePath=$OPTARG ;;
            *) echo ${USAGE}; exit 1 ;;
        esac
    done
        
    # find the existing root directory
    while [ ${thePath} != "." -a ! -d ${thePath} ]
    do
        thePath=`dirname ${thePath}`
    done

    if [ ${thePath} = "." -o ! -d ${thePath} ]
    then
        echo "The path, $2, could not be found."
        echo Return 1
        exit 1
    fi


    # at this point, we have set theType and thePath.
}


getVolume() {
    if [ "${theType}" != "-f" ] && [ "${theType}" != "-b" ]
    then
       return
    fi

 
    case ${OSNAME} in

        SunOS | AIX | Linux)

            # look at space available ($4).  If it is a number, we are on
            # the right line.  Otherwise, the line wraps and we need the
            # one above.
            data=`${DF} -k ${thePath} | ${AWK} '{print $4}' | ${TAIL} -1`
            #echo data=$data
            test ${data} -ge 0 >/dev/null 2>&1
            if [ $? -ne 0 ]
            then
                ${DF} -k ${thePath} | ${AWK} '{print $1}' | ${TAIL} -2 | ${HEAD} -1 >> $LOG
            else
                ${DF} -k ${thePath} | ${AWK} '{print $1}' | ${TAIL} -1 >> $LOG
            fi
            ;;

        HP-UX)
            ${DF} ${thePath} | ${AWK} '{LASTCHAR=substr($2,length($2))
               if(LASTCHAR==")") print substr($2,2,length($2)-2)
               else print substr($2,2)
            }' >> $LOG
            ;;

        *)
         echo "other";;
    esac

    if [ "${theType}" != "-b" ] 
    then 
        echo Return=0
        exit 0
    fi
}


getSpace() {
    case ${OSNAME} in
        SunOS | Linux)
            ${DF} -k ${thePath} | ${TAIL} -1 | ${AWK} '{print $(NF-2)}' >> $LOG ;;
        AIX)
            ${DF} -k ${thePath} | ${AWK} '{print $3}' | ${TAIL} -1  >> $LOG ;;
        HP-UX)
            ${DF} -k ${thePath} | ${HEAD} -2 | ${TAIL} -1 | ${AWK} '{print $1}' >> $LOG ;;
        *)
          echo "other" ;;
    esac

}

setOSParams() {
    OSNAME=`uname -s`
    case ${OSNAME} in
        SunOS)
            DF=/usr/sbin/df
            AWK=/usr/bin/awk
            HEAD=/usr/bin/head
            TAIL=/usr/bin/tail
            ;;
        AIX)
            DF=/usr/bin/df
            AWK=/usr/bin/awk
            HEAD=/usr/bin/head
            TAIL=/usr/bin/tail
            ;;

         HP-UX)
            DF=/usr/bin/df
            AWK=/usr/bin/awk
            HEAD=/usr/bin/head
            TAIL=/usr/bin/tail
            ;;

         Linux)
            DF=/bin/df
            AWK=/usr/bin/awk
            HEAD=/usr/bin/head
            TAIL=/usr/bin/tail
            ;;

          *)
            echo "other"
    esac
}

setOSParams
checkUsage $*
getVolume
getSpace

echo Return=0
exit 0
