#!/bin/ksh
#set -x
#
# IBM Confidential
# OCO Source Materials
# 5647-A01, 5722-SS1
# (C) Copyright IBM Corp. 2009
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#
#
# Get the OS Name.  We have to assume uname is alway available.
OS=`uname -s`

#------- Globals ----------
rc=0

#------- Shell commands -------
# Set up variables based on the OS.
if [[ "${OS}" = "Linux" ]] ; then
  # LINUX
  CUT=/usr/bin/cut
  MV=/bin/mv
  MKDIR=/bin/mkdir
  CHOWN=/bin/chown
  CHMOD=/bin/chmod
  RM=/bin/rm
  DATE=/bin/date
  CD=cd
  LS=ls
  GREP=/bin/grep
  SED=/bin/sed
  WHICH=/usr/bin/which
  WHENCE=whence
  BASENAME=/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/bin/awk
  ENV=env
  XDIR=/usr/bin/X11
  IS64BIT=`uname -m`
elif [[ "${OS}" = "HP-UX" ]] ; then
  # HP-UX
  CUT=/bin/cut
  MV=/bin/mv
  MKDIR=/bin/mkdir
  CHOWN=/bin/chown
  CHMOD=/bin/chmod
  RM=/bin/rm
  DATE=/bin/date
  CD=cd
  LS=/bin/ls
  GREP=/usr/bin/grep
  SED=/usr/bin/sed
  WHICH=/bin/which
  WHENCE=whence
  BASENAME=/usr/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/usr/bin/awk
  ENV=env
  XDIR=/usr/bin/X11
  IS64BIT=`/usr/bin/getconf KERNEL_BITS`
  HP_HWCHECK=`uname -m`
  set +u
elif [[ "${OS}" = "SunOS" ]] ; then
  # Solaris
  CUT=/usr/bin/cut
  MV=/usr/bin/mv
  MKDIR=/usr/bin/mkdir
  CHOWN=/usr/bin/chown
  CHMOD=/usr/bin/chmod
  RM=/usr/bin/rm
  DATE=/usr/bin/date
  CD=cd
  ENV=env
  LS=/usr/bin/ls
  GREP=/usr/bin/grep
  SED=/usr/bin/sed
  WHICH=/usr/bin/which
  WHENCE=whence
  BASENAME=/usr/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/usr/bin/awk
  XDIR=/usr/openwin/bin
  IS64BIT=`/usr/bin/isainfo -b`
elif [[ "${OS}" = "AIX" || "${OS}" = "OS400" ]] ; then
  # AIX or OS/400 PACE
  CUT=/usr/bin/cut
  MV=/usr/bin/mv
  MKDIR=/usr/bin/mkdir
  CHOWN=/usr/bin/chown
  CHMOD=/usr/bin/chmod
  RM=/usr/bin/rm
  DATE=/usr/bin/date
  CD=cd
  ENV=env
  LS=/usr/bin/ls
  GREP=/usr/bin/grep
  SED=/usr/bin/sed
  WHICH=/usr/bin/which
  WHENCE=whence
  BASENAME=/usr/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/usr/bin/awk
  XDIR=/usr/bin/X11
  AIXVERSION=`uname -v`
  AIXRELEASE=`uname -r`
#  fi
else
  # We have an unsupported platform
  # Print a message and exit.
  echo "Unsupported platform"
  echo Return 1
  exit 1
fi





USAGE()
{
    echo " " 
    echo "Usage: start_stop_db2.sh db2instance_owner_home tklm_instance_name action"
    echo "       [ -h | -? ]"
    echo " " 
    echo "where:"
    echo " " 
    echo " <tklm_v2_location>         Specifies the TKLM V2 location"
    echo " <db2_9.1_location>         Specifies the DB2 version 9.1 location for the migration"
    echo " <db2_9.7_location>         Specifies the DB2 version 9.7 location for the migration"
    echo " <tklm_instance_name>       Specifies the name of the TKLM V1 instance. "
    echo " <dbname>                   Specifies the name of the database"
    echo " <username>                 Specifies the DB2 instance owner name"
    echo " <password>                 Specifies the DB2 instance owner password"
    echo " -h | -?                    Prints the usage information."
    echo " "
}

ParseCmdLine()
{
   # Input   : reqires the following environment variables
    #           1) $*

    if [[ "$#" != 4 ]];then
        USAGE
        echo Return 1
        exit 1
    fi

    export TKLM_HOME=$1
    export DB2_HOME=$2
    export DB2_INSTANCE=$3
    export ACTION=$4
}

prepare_db2_start_dbm()
{
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 start database manager\"" >>${START_STOP_LOG}
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 start database manager" >>${START_STOP_LOG}
    rc=$?
    echo "rc=$rc" >> ${START_STOP_LOG}
    if [[ "$rc" = "0" ]]; then  
        # " database manager started successfully" 
        echo ""
    else 
        if [[ "$rc" -lt "5" ]] ; then 
            # " WARNING: The database manager is already active" 
            rc = 0;
        else 
            # " ERROR: database manager failed to start"
        echo "Return 11" >> ${START_STOP_LOG}
            echo Return 11
	    exit 11
        fi
    fi

}

prepare_db2_stop_dbm()
{

    #  db2 terminate connection
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile;db2 terminate\"" >>${START_STOP_LOG}
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile;db2 terminate" >>${START_STOP_LOG}
    rc=$?
    echo "rc=$rc" >> ${START_STOP_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " db terminate succedded" 
        echo ""
    else 
        # " db terminate failed"
        echo "Return 27" >> ${START_STOP_LOG}
        echo Return 27 
        exit 27
    fi 

    #  force applications all
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 force application all\"" >>$START_STOP_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile;db2 force application all" >>$START_STOP_LOG
    rc=$?
    echo "rc=$rc" >> ${START_STOP_LOG} 
    if [[ "$rc" = "0" ]] ; then  
         # " db force application succeeded" 
        echo ""
    else 
        # " db force application failed"
        echo "Return 25" >> ${START_STOP_LOG}
        echo "Return 25"
        exit 25
    fi

    # stop the database manager
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2stop force\"" >>$START_STOP_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2stop force" >>$START_STOP_LOG
    rc=$?
    echo "rc=$rc" >> ${START_STOP_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " database manager stopped successfully" 
        echo ""
    else 
        echo "Return 18" >> ${START_STOP_LOG}
        echo Return 18
        exit 18
    fi

    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile;db2_kill\"" >>$START_STOP_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile;db2_kill" >>$START_STOP_LOG

}

prepare_db2_restart_dbm()
{

    prepare_db2_stop_dbm

    prepare_db2_start_dbm
    
}



############################################################################
#  Main code starts here
############################################################################

  
  

  #Parse command line arguments
  ParseCmdLine $*
export START_STOP_LOG=$TKLM_HOME/migration/migrate_db2server.log

    
  if [ $DB2_INSTANCE ] ; then
    echo ""
    # " DB2 instance name: $DB2_INSTANCE"
  else
    # " DB2 instance name not found"
    echo "Return 5" >> ${START_STOP_LOG}
    echo Return 5 
    exit 5 
  fi
  if [ $TKLM_HOME ] ; then
    echo ""
    # " TKLM_HOME defined
  else
    # " TKLM_HOME not defined
    echo "Return 10" >> ${START_STOP_LOG}
    exit 10 
  fi
  if [ $DB2_HOME ] ; then
    echo ""
    # " DB2_HOME defined
  else
    # " DB2_HOME not defined
    echo "Return 15" >> ${START_STOP_LOG}
    exit 15 
  fi
  if [ $ACTION ] ; then
    echo ""
    # " ACTION defined
  else
    # " ACTION not defined
    echo "Return 20" >> ${START_STOP_LOG}
    exit 20 
  fi

if [ $ACTION = 1 ]; then  
       prepare_db2_start_dbm
else
       prepare_db2_stop_dbm
fi



echo " "
echo Return=0
exit 0
