#!/bin/ksh
# Licensed Materials-Property of IBM
# (C)Copyright IBM Corp 2009
# All rights reserved
# US Government Users Restricted Rights
# Use, duplication, or disclosure restricted
# by GSA ADP Schedule Contract with IBM Corp
#
#
#
# Get the OS Name.  We have to assume uname is alway available.
OS=`uname -s`

#------- Globals ----------
rc=0

#------- Shell commands -------
# Set up variables based on the OS.
if [[ "${OS}" = "Linux" ]] ; then
  # LINUX
  CUT=/usr/bin/cut
  MV=/bin/mv
  MKDIR=/bin/mkdir
  CHOWN=/bin/chown
  CHMOD=/bin/chmod
  RM=/bin/rm
  DATE=/bin/date
  CD=cd
  LS=ls
  GREP=/bin/grep
  SED=/bin/sed
  WHICH=/usr/bin/which
  WHENCE=whence
  BASENAME=/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/bin/awk
  ENV=env
  XDIR=/usr/bin/X11
  IS64BIT=`uname -m`
elif [[ "${OS}" = "HP-UX" ]] ; then
  # HP-UX
  CUT=/bin/cut
  MV=/bin/mv
  MKDIR=/bin/mkdir
  CHOWN=/bin/chown
  CHMOD=/bin/chmod
  RM=/bin/rm
  DATE=/bin/date
  CD=cd
  LS=/bin/ls
  GREP=/usr/bin/grep
  SED=/usr/bin/sed
  WHICH=/bin/which
  WHENCE=whence
  BASENAME=/usr/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/usr/bin/awk
  ENV=env
  XDIR=/usr/bin/X11
  IS64BIT=`/usr/bin/getconf KERNEL_BITS`
  HP_HWCHECK=`uname -m`
  set +u
elif [[ "${OS}" = "SunOS" ]] ; then
  # Solaris
  CUT=/usr/bin/cut
  MV=/usr/bin/mv
  MKDIR=/usr/bin/mkdir
  CHOWN=/usr/bin/chown
  CHMOD=/usr/bin/chmod
  RM=/usr/bin/rm
  DATE=/usr/bin/date
  CD=cd
  ENV=env
  LS=/usr/bin/ls
  GREP=/usr/bin/grep
  SED=/usr/bin/sed
  WHICH=/usr/bin/which
  WHENCE=whence
  BASENAME=/usr/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/usr/bin/awk
  XDIR=/usr/openwin/bin
  IS64BIT=`/usr/bin/isainfo -b`
elif [[ "${OS}" = "AIX" || "${OS}" = "OS400" ]] ; then
  # AIX or OS/400 PACE
  CUT=/usr/bin/cut
  MV=/usr/bin/mv
  MKDIR=/usr/bin/mkdir
  CHOWN=/usr/bin/chown
  CHMOD=/usr/bin/chmod
  RM=/usr/bin/rm
  DATE=/usr/bin/date
  CD=cd
  ENV=env
  LS=/usr/bin/ls
  GREP=/usr/bin/grep
  SED=/usr/bin/sed
  WHICH=/usr/bin/which
  WHENCE=whence
  BASENAME=/usr/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/usr/bin/awk
  XDIR=/usr/bin/X11
  AIXVERSION=`uname -v`
  AIXRELEASE=`uname -r`
#  fi
else
  # We have an unsupported platform
  # Print a message and exit.
  echo "Unsupported platform"
  exit 1
fi

export TKLMDBMIGR_LOG=$1/migration/migrate_tklmdbmigr.log
export TKLMDBMIGR_TMP=$1/migration/tklmdbmigr.tmp.$$



get_DB2_Version ()
{

    # Input   : reqires the following environment variables
    #           1) DB2_INSTANCE
    #           2) DB2N
    # Output  : sets the following environment variables 
    #           1) DB2V9PATH
    #           2) DB2VER

    export DB2INSTANCE=${DB2_INSTANCE}



#          DB2VERSION=`${DB2NLOC}/bin/db2level | $GREP "DB2 v" | cut -f2 -d'"' | cut -f2 -d' ' | cut -c2-`
          DB2VERSION=`${DB2NLOC}/install/db2ls | $GREP "$DB2NLOC" | $AWK '{print $2}'`
          TOK1=`echo ${DB2VERSION} | cut -f1 -d.`
          TOK2=`echo ${DB2VERSION} | cut -f2 -d.`

          if [[ "$TOK1$TOK2" = "97" ]] ; then
              DB2VER=97
          else
             if [[ "$TOK1$TOK2" = "95" ]]; then
               DB2VER=95
             else
               echo "rc=100" >> ${TKLMDBMIGR_LOG}
               exit 100
             fi
          fi
    export DB2INSTANCE=
    
}

USAGE()
{
    echo " " 
    echo "Usage: tklmdbmigr tklm_v2_location db2_9.1_location db2_9.7_location tklm_instance_name dbname dbusername dbpassword"
    echo "       [ -h | -? ]"
    echo " " 
    echo "where:"
    echo " " 
    echo " <tklm_v2_location>         Specifies the TKLM V2 location"
    echo " <db2_9.1_location>         Specifies the DB2 version 9.1 location for the migration"
    echo " <db2_9.7_location>         Specifies the DB2 version 9.7 location for the migration"
    echo " <tklm_instance_name>       Specifies the name of the TKLM V1 instance. "
    echo " <dbname>                   Specifies the name of the database"
    echo " <username>                 Specifies the DB2 instance owner name"
    echo " <password>                 Specifies the DB2 instance owner password"
    echo " -h | -?                    Prints the usage information."
    echo " "
}

ParseCmdLine()
{
   # Input   : reqires the following environment variables
    #           1) $*
    # Output  : sets the following environment variables 
    #           1) TDSI
    #           2) DB2N

    if [[ "$#" != 8 ]];then
        USAGE
        exit 1
    fi

    export TKLM_HOME=$1
    export DB2OLOC=$2 # 9.1 location
    export DB2NLOC=$3 # 9.7 or 9.5 location 
    export DB2_INSTANCE=$4
    export DB2_DBNAME=$5
    export DB2_USERNAME=$4
    export DB2_PWD=$6
    export DB2_HOME=$7
    export ACTION=$8
    export SVCE_NAME=DB2_${DB2_INSTANCE}
}

prepare_db2_start_dbm()
{
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 start database manager\"" >>${TKLMDBMIGR_LOG}
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 start database manager" >>${TKLMDBMIGR_LOG}
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
    if [[ "$rc" = "0" ]]; then  
        # " database manager started successfully" 
        echo ""
    else 
        if [[ "$rc" = "4" ]] ; then 
            # " WARNING: The database manager is already active" 
            rc = 0;
        else 
            # " ERROR: database manager failed to start"
        echo "Return 11" >> ${TKLMDBMIGR_LOG}
	    exit 11
        fi
    fi

}

prepare_db2_stop_dbm()
{

    #  db2 terminate connection
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile;db2 terminate\"" >>${TKLMDBMIGR_LOG}
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile;db2 terminate" >>${TKLMDBMIGR_LOG}
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " db terminate succedded" 
        echo ""
    else 
        # " db terminate failed"
        echo "Return 27" >> ${TKLMDBMIGR_LOG}
        exit 27
    fi 

    #  force applications all
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 force application all\"" >>$TKLMDBMIGR_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile;db2 force application all" >>$TKLMDBMIGR_LOG
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
         # " db force application succeeded" 
        echo ""
    else 
        # " db force application failed"
        echo "Return 25" >> ${TKLMDBMIGR_LOG}
        exit 25
    fi

    # stop the database manager
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2stop force\"" >>$TKLMDBMIGR_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2stop force" >>$TKLMDBMIGR_LOG
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " database manager stopped successfully" 
        echo ""
    else 
        echo "Return 18" >> ${TKLMDBMIGR_LOG}
        exit 18
    fi

    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile;db2_kill\"" >>$TKLMDBMIGR_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile;db2_kill" >>$TKLMDBMIGR_LOG

}

deactivate_db ()
{

    
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 deactivate db $DB2_DBNAME \"" >>${TKLMDBMIGR_LOG}
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 deactivate db $DB2_DBNAME" >>${TKLMDBMIGR_LOG}
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " db deactivation succeded
        echo ""
    else 
        if [[ "$rc" = "2" ]]; then
           rc=0
        else
           # " db terminate failed"
           echo "Return 30" >> ${TKLMDBMIGR_LOG}
           exit 30 
       fi
    fi 

}

ckmigrate_db ()
{

    echo DB2VER=$DB2VER
    if [ "$DB2VER" = "97" ]; then 
        CKBIN=$DB2NLOC/bin/db2ckupgrade
    else
        CKBIN=$DB2NLOC/bin/db2ckmig
    fi
    #  db2 terminate connection
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; $CKBIN $DB2_DBNAME -l ~/db2ckmig.log\"" >> ${TKLMDBMIGR_LOG} 
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; $CKBIN $DB2_DBNAME -l ~/db2ckmig.log" >> ${TKLMDBMIGR_LOG}
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " db2ckmig succeded.
        echo ""
    else 
        # " db2ckmig succeded.
        echo "Return 40" >> ${TKLMDBMIGR_LOG}
        exit 40 
    fi 

}
prepare_db2_restart_dbm()
{

    prepare_db2_stop_dbm

    prepare_db2_start_dbm
    
}


migrate_db()
{

    # Input   : reqires the following environment variables
    #           2) DB2V9PATH
    #           3) DB2_INSTANCE
    #           4) DB2_DBNAME
    # Output  : sets the following environment variables 
    #           1) ERROR
    # Process : Following tasks are performed during migration 
    #           1) migrate the DB2 instance from DB2 9 directory
    #           2) start the DB2 9.1 database manager 
    #           3) migrate the DB2 database 

    echo " "
    
    # " Starting the migration task .."

    DB2INSTANCE=$DB2_INSTANCE
    rc=0

        echo "migrate_db" >> ${TKLMDBMIGR_LOG}
        echo "ACTION=$ACTION" >> ${TKLMDBMIGR_LOG}
    if [[ "$ACTION" = "migrateDb" ]]; then

            prepare_db2_start_dbm
    
        # migrate the database
        echo "su - ${DB2_INSTANCE} -c \". $DB2_HOME/sqllib/db2profile; db2 migrate database ${DB2_DBNAME}\"" >> ${TKLMDBMIGR_LOG}
        su - ${DB2_INSTANCE} -c ". $DB2_HOME/sqllib/db2profile; db2 migrate database ${DB2_DBNAME}" >> ${TKLMDBMIGR_LOG}
        rc=$?
        echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
        # rc=2 is returned if the database is alredy at the same level
        if [[ "$rc" = "0" || "$rc" = "2" ]] ; then  
            # " database migrated successfully" 
            echo ""
        else 
            echo "Return 60" >> ${TKLMDBMIGR_LOG}
            exit 60 
        fi
        
        prepare_db2_stop_dbm
    fi 

}


post_migration()
{

    # Input   : reqires the following environment variables
    #           1) DB2V9PATH
    #           2) DB2_INSTANCE
    #           3) DB2_DBNAME
    # Output  : sets the following environment variables 
    #           1) ERROR
    # Process : Following tasks are performed during pre-migration 
    #           1) rebind the DB2 packages
    #           2) start the database manager
    #           3) reset the DIAGLEVEL
    if [[ "$ACTION" = "setServerConfig" ]]; then 
	
       echo " "
       echo post_migration >> $TKLMDBMIGR_LOG 
       # " Starting the post migration task .."
   
       DB2INSTANCE=$DB2_INSTANCE
       export RBIND_LOGFILE=$DB2_DBNAME.log
   
       # rebind the DB2 database packages
       echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2rbind $DB2_DBNAME -l $RBIND_LOGFILE\"" >>$TKLMDBMIGR_LOG
       su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2rbind $DB2_DBNAME -l $RBIND_LOGFILE" >>$TKLMDBMIGR_LOG
       rc=$?
       echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
       if [[ "$rc" = "0" ]] ; then  
           # "db2rbind succeded" 
           echo ""
       else 
           # "db2rbind failed" 
   	exit 70 
       fi
   
       prepare_db2_start_dbm
   
       # update the DIAGLEVEL
       echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 UPDATE DBM CFG USING DIAGLEVEL 3\"" >>$TKLMDBMIGR_LOG
       echo su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 UPDATE DBM CFG USING DIAGLEVEL 3" >>$TKLMDBMIGR_LOG
       rc=$?
       echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
       if [[ "$rc" = "0" ]] ; then  
           echo ""
           # " update dbm cfg succeded" 
       else 
           echo "Return 75" >> ${TKLMDBMIGR_LOG}
           exit 75 
       fi
       # turn off health_mon
       echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 UPDATE DBM CFG USING HEALTH_MON OFF\"" >>$TKLMDBMIGR_LOG
       su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 UPDATE DBM CFG USING HEALTH_MON OFF" >>$TKLMDBMIGR_LOG
       rc=$?
       echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
       if [[ "$rc" = "0" ]] ; then  
           echo ""
           # " update dbm cfg health_mon succeded" 
       else 
           echo "Return 76" >> ${TKLMDBMIGR_LOG}
           exit 76
       fi
   
    #update the secondary log to 25
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 UPDATE DB CFG FOR $DB2_DBNAME USING LOGSECOND 25 \"" >>$TKLMDBMIGR_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 UPDATE DB CFG FOR $DB2_DBNAME USING LOGSECOND 25 " >>$TKLMDBMIGR_LOG
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
    if [[ "$rc" = "0" ]] ; then
        # " update db cfg for db succeded"
        echo ""
    else
        echo "Return 76" >> ${TKLMDBMIGR_LOG}
        exit 76
    fi

   
       # list database directory
       echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 list db directory\"" >>$TKLMDBMIGR_LOG
       su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 list db directory" >>$TKLMDBMIGR_LOG
       rc=$?
       echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
       if [[ "$rc" = "0" ]] ; then  
           echo ""
           # " list db directory succedded" 
       else 
           # "ERROR: list db directory failed" 
           echo "Return 80" >> ${TKLMDBMIGR_LOG}
           exit 80 
       fi
   
       #  connect to the database
       echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 connect to $DB2_DBNAME\"" >>$TKLMDBMIGR_LOG
       su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 connect to $DB2_DBNAME" >>$TKLMDBMIGR_LOG
       rc=$?
       echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
       if [[ "$rc" = "0" ]] ; then  
           echo ""
           # " db connection succedded" 
       else 
           # "ERROR: db connection failed"
           echo "Return 85" >> ${TKLMDBMIGR_LOG}
           exit 85 
       fi
   
       prepare_db2_stop_dbm
    fi
    
}

test_DB2_INSTANCE(){
   # Function to check if the database instance associated
   # with a given directory server instance is at  
   # a supported level for pre-migration tasks to be performed.

   # Redirect the output of db2ilist to a temporary file
   $DB2OLOC/instance/db2ilist > ${TKLMDBMIGR_TMP}

   # Grep to see if the instance name is present in the list
   ${GREP} $DB2_INSTANCE ${TKLMDBMIGR_TMP} > /dev/null 2>&1
   if [[ $? -eq 0 ]] ; then
      NEED_TO_MIGRATE_DB2_INST="false"
   else
      NEED_TO_MIGRATE_DB2_INST="true"
   fi
   
   #Delete the tmp file created
   ${RM} -rf ${TKLMDBMIGR_TMP}
	       
}

set_comm (){
echo su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2set DB2COMM=tcpip\" >> ${TKLMDBMIGR_LOG}
su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2set DB2COMM=tcpip" >> ${TKLMDBMIGR_LOG}
rc=$?
echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
}
set_svcename (){
echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 update dbm cfg USING SVCENAME DB2_${DB2_INSTANCE}\"" >> ${TKLMDBMIGR_LOG}
su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 update dbm cfg USING SVCENAME DB2_${DB2_INSTANCE}" >> ${TKLMDBMIGR_LOG}
rc=$?
echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
}

############################################################################
#  Main code starts here
############################################################################

  
  

  echo ============$0========== > $1/migration/migrate_tklmdbmigr.log
  #Parse command line arguments
  ParseCmdLine $*
 

    
  if [ $DB2_INSTANCE ] ; then
    echo ""
    # " DB2 instance name: $DB2_INSTANCE"
  else
    # " DB2 instance name not found"
    echo "Return 5" >> ${TKLMDBMIGR_LOG}
    exit 5 
  fi

  if [ $DB2_DBNAME ] ; then
    echo ""
    # " DB2 database name: $DB2_DBNAME"
  else
    # " DB2 database name not found"
    echo "Return 10" >> ${TKLMDBMIGR_LOG}
    exit 10 
  fi

  if [ $DB2NLOC ] ; then
    if [[ -r $DB2NLOC ]] ; then
        # " DB2 Install Location: $DB2V97PATH"
        # DB2 Version 
        echo ""
    else
        # " ERROR: Invalid DB2 Install Location specified
        echo "Return 15" >> ${TKLMDBMIGR_LOG}
        exit 15 
    fi
  else
    # " ERROR: Invalid DB2 Install Location specified
    echo "Return 15" >> ${TKLMDBMIGR_LOG}
    exit 15 
  fi

  if [ $DB2OLOC ] ; then
    if [[ -r $DB2OLOC ]] ; then
        echo ""
        # " DB2 Install Location: $DB2V9PATH"
        # DB2 Version 
    else
        # " ERROR: Invalid DB2 Install Location specified
        echo "Return 20" >> ${TKLMDBMIGR_LOG}
        exit 20 
    fi
  else
    # " ERROR: Invalid DB2 Install Location specified
    echo "Return 20" >> ${TKLMDBMIGR_LOG}
    exit 20 
  fi

test_DB2_INSTANCE
set_comm
set_svcename

if [[ "$ACTION" =  "migrateDb" ]]; then
    echo calling migrate_db >> ${TKLMDBMIGR_LOG}
    migrate_db
fi
post_migration

echo "Return=0"
exit 0
