#!/bin/ksh
#set -x
#
# IBM Confidential
# OCO Source Materials
# 5647-A01, 5722-SS1
# (C) Copyright IBM Corp. 2009
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#
#
# Get the OS Name.  We have to assume uname is alway available.
OS=`uname -s`

#------- Globals ----------
rc=0

#------- Shell commands -------
# Set up variables based on the OS.
if [[ "${OS}" = "Linux" ]] ; then
  # LINUX
  CUT=/usr/bin/cut
G
  MV=/bin/mv
  MKDIR=/bin/mkdir
  CHOWN=/bin/chown
  CHMOD=/bin/chmod
  RM=/bin/rm
  DATE=/bin/date
  CD=cd
  LS=ls
  GREP=/bin/grep
  SED=/bin/sed
  WHICH=/usr/bin/which
  WHENCE=whence
  BASENAME=/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/bin/awk
  ENV=env
  XDIR=/usr/bin/X11
  IS64BIT=`uname -m`
elif [[ "${OS}" = "HP-UX" ]] ; then
  # HP-UX
  CUT=/bin/cut
  MV=/bin/mv
  MKDIR=/bin/mkdir
  CHOWN=/bin/chown
  CHMOD=/bin/chmod
  RM=/bin/rm
  DATE=/bin/date
  CD=cd
  LS=/bin/ls
  GREP=/usr/bin/grep
  SED=/usr/bin/sed
  WHICH=/bin/which
  WHENCE=whence
  BASENAME=/usr/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/usr/bin/awk
  ENV=env
  XDIR=/usr/bin/X11
  IS64BIT=`/usr/bin/getconf KERNEL_BITS`
  HP_HWCHECK=`uname -m`
  set +u
elif [[ "${OS}" = "SunOS" ]] ; then
  # Solaris
  CUT=/usr/bin/cut
  MV=/usr/bin/mv
  MKDIR=/usr/bin/mkdir
  CHOWN=/usr/bin/chown
  CHMOD=/usr/bin/chmod
  RM=/usr/bin/rm
  DATE=/usr/bin/date
  CD=cd
  ENV=env
  LS=/usr/bin/ls
  GREP=/usr/bin/grep
  SED=/usr/bin/sed
  WHICH=/usr/bin/which
  WHENCE=whence
  BASENAME=/usr/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/usr/bin/awk
  XDIR=/usr/openwin/bin
  IS64BIT=`/usr/bin/isainfo -b`
elif [[ "${OS}" = "AIX" || "${OS}" = "OS400" ]] ; then
  # AIX or OS/400 PACE
  CUT=/usr/bin/cut
  MV=/usr/bin/mv
  MKDIR=/usr/bin/mkdir
  CHOWN=/usr/bin/chown
  CHMOD=/usr/bin/chmod
  RM=/usr/bin/rm
  DATE=/usr/bin/date
  CD=cd
  ENV=env
  LS=/usr/bin/ls
  GREP=/usr/bin/grep
  SED=/usr/bin/sed
  WHICH=/usr/bin/which
  WHENCE=whence
  BASENAME=/usr/bin/basename
  DIRNAME=/usr/bin/dirname
  AWK=/usr/bin/awk
  XDIR=/usr/bin/X11
  AIXVERSION=`uname -v`
  AIXRELEASE=`uname -r`
#  fi
else
  # We have an unsupported platform
  # Print a message and exit.
  echo "Unsupported platform"
  exit 1
fi

export TKLMDBMIGR_LOG=$1/migration/migrate_tklminstmigr.log
export TKLMDBMIGR_TMP=$1/migration/migrate_tklminstmigr.tmp.$$



get_DB2_Version ()
{

    # Input   : reqires the following environment variables
    #           1) DB2_INSTANCE
    #           2) DB2N
    # Output  : sets the following environment variables 
    #           1) DB2V9PATH
    #           2) DB2VER

    export DB2INSTANCE=${DB2_INSTANCE}



#          DB2VERSION=`${DB2NLOC}/bin/db2level | $GREP "DB2 v" | cut -f2 -d'"' | cut -f2 -d' ' | cut -c2-`
          export DB2VERSION=`${DB2NLOC}/install/db2ls | grep "$DB2NLOC" | $AWK '{print $2}'`
          echo "DB2Version=" $DB2VERSION
          TOK1=`echo ${DB2VERSION} | cut -f1 -d.`
          TOK2=`echo ${DB2VERSION} | cut -f2 -d.`

          if [[ "$TOK1$TOK2" = "97" ]] ; then
              DB2VER=97
          else
             if [[ "$TOK1$TOK2" = "95" ]]; then
               DB2VER=95
             else
               echo "Return 100" >> ${TKLMDBMIGR_LOG}
               exit 100
             fi
          fi
    export DB2INSTANCE=
    
}

USAGE()
{
    echo " " 
    echo "Usage: tklmdbmigr tklm_v2_location db2_9.1_location db2_9.7_location tklm_instance_name dbname dbusername dbpassword"
    echo "       [ -h | -? ]"
    echo " " 
    echo "where:"
    echo " " 
    echo " <tklm_v2_location>         Specifies the TKLM V2 location"
    echo " <db2_9.1_location>         Specifies the DB2 version 9.1 location for the migration"
    echo " <db2_9.7_location>         Specifies the DB2 version 9.7 location for the migration"
    echo " <tklm_instance_name>       Specifies the name of the TKLM V1 instance. "
    echo " <dbname>                   Specifies the name of the database"
    echo " <username>                 Specifies the DB2 instance owner name"
    echo " <password>                 Specifies the DB2 instance owner password"
    echo " -h | -?                    Prints the usage information."
    echo " "
}

ParseCmdLine()
{
   # Input   : reqires the following environment variables
    #           1) $*
    # Output  : sets the following environment variables 
    #           1) TDSI
    #           2) DB2N

    if [[ "$#" != 7 ]];then
        USAGE
        echo Return 1
        exit 1
    fi

    export TKLM_HOME=$1
    export DB2OLOC=$2 # 9.1 location
    export DB2NLOC=$3 # 9.7 or 9.5 location 
    export DB2_INSTANCE=$4
    export DB2_USERNAME=$4
    export DB2_DBNAME=$5
    export DB2_PWD=$6
    export DB2_HOME=$7
    export SVCE_NAME=DB2_${DB2_INSTANCE}
}

prepare_db2_start_dbm()
{
    echo "su - $DB2_INSTANCE -c \". $DB2_DBNAME/sqllib/db2profile; db2 start database manager\"" >>${TKLMDBMIGR_LOG}
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 start database manager" >>${TKLMDBMIGR_LOG}
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
    if [[ "$rc" = "0" ]]; then  
        # " database manager started successfully" 
        echo ""
    else 
        if [[ "$rc" -le "4" ]] ; then 
            # " WARNING: The database manager is already active" 
            rc=0;
        else 
            # " ERROR: database manager failed to start"
        echo "Return 11" >> ${TKLMDBMIGR_LOG}
	    exit 11
        fi
    fi

}

prepare_db2_stop_dbm()
{

    #  db2 terminate connection
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile;db2 terminate\"" >>${TKLMDBMIGR_LOG}
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile;db2 terminate" >>${TKLMDBMIGR_LOG}
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " db terminate succedded" 
        echo ""
    else 
        # " db terminate failed"
        echo "Return 27" >> ${TKLMDBMIGR_LOG}
        exit 27
    fi 

    #  force applications all
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 force application all\"" >>$TKLMDBMIGR_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile;db2 force application all" >>$TKLMDBMIGR_LOG
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
         # " db force application succeeded" 
        echo ""
    else 
        # " db force application failed"
        echo "Return 25" >> ${TKLMDBMIGR_LOG}
        exit 25
    fi

    # stop the database manager
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2stop force" >>$TKLMDBMIGR_LOG
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " database manager stopped successfully" 
        echo ""
    else 
        echo "Return 18" >> ${TKLMDBMIGR_LOG}
        exit 18
    fi

    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile;db2_kill\"" >>$TKLMDBMIGR_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile;db2_kill" >>$TKLMDBMIGR_LOG

}

deactivate_db ()
{

    
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 deactivate db $DB2_DBNAME \"" >>${TKLMDBMIGR_LOG}
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 deactivate db $DB2_DBNAME" >>${TKLMDBMIGR_LOG}
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " db deactivation succeded
        echo ""
    else 
        if [[ "$rc" = "2" ]]; then
           rc=0
        else
           # " db deactivate failed"
           echo "Return 30" >> ${TKLMDBMIGR_LOG}
           exit 30 
       fi
    fi 

}

ckmigrate_db ()
{

    echo DB2VER=$DB2VER
    if [ "$DB2VER" = "97" ]; then 
        CKBIN=$DB2NLOC/bin/db2ckupgrade
    else
        CKBIN=$DB2NLOC/bin/db2ckmig
    fi
    prepare_db2_start_dbm
    #  db2 check migrate
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; $CKBIN $DB2_DBNAME -l ~/db2ckmig.log\"" >> ${TKLMDBMIGR_LOG} 
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; $CKBIN $DB2_DBNAME -l ~/db2ckmig.log" >> ${TKLMDBMIGR_LOG}
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " db2ckmig succeded.
        echo ""
    else 
        # " db2ckmig failed
        echo "Return 40" >> ${TKLMDBMIGR_LOG}
        exit 40 
    fi 
    prepare_db2_stop_dbm

}
prepare_db2_restart_dbm()
{

    prepare_db2_stop_dbm

    prepare_db2_start_dbm
    
}


pre_migration()
{

    # Input   : reqires the following environment variables
    #           1) DB2V9PATH
    #           2) DB2_INSTANCE
    #           3) DB2_DBNAME
    #           which is picked up from the properties file
    # Output  : sets the following environment variables 
    #           1) ERRORLEVEL 
    #           2) ERROR
    # Process : Following tasks are performed during pre-migration 
    #           1) start the database manager for DB2_INSTANCE
    #           2) connect to the DB2 database DB2_DBNAME
    #           3) update the DBM CFG to DIAGLEVEL 3
    #           4) terminate all connections to the DB2 database DB2_DBNAME

	
    set DB2INSTANCE=${DB2_INSTANCE}
    rc=0

    echo " "

    prepare_db2_start_dbm
    
    # list database directory
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 list db directory\"" >>$TKLMDBMIGR_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 list db directory" >>$TKLMDBMIGR_LOG
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" -le "4" ]] ; then  
        # " list db directory succedded" 
        echo ""
    else 
        # " list db directory failed" 
        echo "Return 14" >> ${TKLMDBMIGR_LOG}
        exit 14
    fi 

    #  connect to the database
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 connect to $DB2_DBNAME\"" >>$TKLMDBMIGR_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 connect to $DB2_DBNAME" >>$TKLMDBMIGR_LOG
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" -le "4" ]] ; then  
        # " db connection succedded" 
        echo ""
    else 
        # " db connection failed"
        echo "Return 16" >> ${TKLMDBMIGR_LOG}
        exit 16
    fi

       
    # update the DIAGLEVEL level
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 UPDATE DBM CFG USING DIAGLEVEL 3\"" >>$TKLMDBMIGR_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 UPDATE DBM CFG USING DIAGLEVEL 3" >>$TKLMDBMIGR_LOG
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " update dbm cfg succeded" 
        echo ""
    else 
        echo "Return 29" >> ${TKLMDBMIGR_LOG}
        exit 29
    fi

    #update the secondary log to -1
    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 UPDATE DB CFG FOR $DB2_DBNAME USING LOGSECOND -1 \"" >>$TKLMDBMIGR_LOG
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2 UPDATE DB CFG FOR $DB2_DBNAME USING LOGSECOND -1 " >>$TKLMDBMIGR_LOG
    rc=$?
    echo "rc=$rc" >> ${TKLMDBMIGR_LOG} 
    if [[ "$rc" = "0" ]] ; then  
        # " update db cfg for db succeded" 
        echo ""
    else 
        echo "Return 31" >> ${TKLMDBMIGR_LOG}
        exit31 
    fi


    echo "su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile;db2 terminate\"" >>${TKLMDBMIGR_LOG}
    su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile;db2 terminate" >>${TKLMDBMIGR_LOG}
    deactivate_db
    prepare_db2_stop_dbm


}

test_DB2_INSTANCE(){
   # Function to check if the database instance associated
   # with a given directory server instance is at  
   # a supported level for pre-migration tasks to be performed.

   # Redirect the output of db2ilist to a temporary file
   $DB2OLOC/instance/db2ilist > ${TKLMDBMIGR_TMP}

   # Grep to see if the instance name is present in the list
   ${GREP} $DB2_INSTANCE ${TKLMDBMIGR_TMP} > /dev/null 2>&1
   if [[ $? -eq 0 ]] ; then
      NEED_TO_MIGRATE_DB2_INST="false"
   else
      NEED_TO_MIGRATE_DB2_INST="true"
   fi
   
   #Delete the tmp file created
   ${RM} -rf ${TKLMDBMIGR_TMP}
	       
}


migrate_instance(){ 
if [ "$DB2VER" = "97" ]; then 
    MIGBIN=$DB2NLOC/instance/db2iupgrade
else
    MIGBIN=$DB2NLOC/instance/db2imigr
fi
echo "$MIGBIN -u $DB2_INSTANCE $DB2_INSTANCE" >> ${TKLMDBMIGR_LOG}
$MIGBIN -u $DB2_INSTANCE $DB2_INSTANCE >> ${TKLMDBMIGR_LOG}
rc=$?
 echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
 if [[ "$rc" = "0" ]] ; then  
      echo ""
      # " migrate instance migration succeded.
 else 
      #ERROR:  migrate instance failed
      echo "Return 50" >> ${TKLMDBMIGR_LOG}
      exit 50 
 fi

}
set_comm (){
echo su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2set DB2COMM=tcpip\" >> ${TKLMDBMIGR_LOG}
su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2set DB2COMM=tcpip" >> ${TKLMDBMIGR_LOG}
rc=$?
echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
}
set_svcename (){
echo su - $DB2_INSTANCE -c \". $DB2_HOME/sqllib/db2profile; db2 update dbm cfg USING SVCENAME $SVCE_NAME\" >> ${TKLMDBMIGR_LOG}
su - $DB2_INSTANCE -c ". $DB2_HOME/sqllib/db2profile; db2set DB2COMM=tcpip" >> ${TKLMDBMIGR_LOG}
rc=$?
echo "rc=$rc" >> ${TKLMDBMIGR_LOG}
}

############################################################################
#  Main code starts here
############################################################################

  
  

  #Parse command line arguments
  ParseCmdLine $*

    
  if [ $DB2_INSTANCE ] ; then
    echo ""
    # " DB2 instance name: $DB2_INSTANCE"
  else
    # " DB2 instance name not found"
    echo "Return 5" >> ${TKLMDBMIGR_LOG}
    exit 5 
  fi

  if [ $DB2_DBNAME ] ; then
    echo ""
    # " DB2 database name: $DB2_DBNAME"
  else
    # " DB2 database name not found"
    echo "Return 10" >> ${TKLMDBMIGR_LOG}
    exit 10 
  fi

  if [ $DB2NLOC ] ; then
    if [[ -r $DB2NLOC ]] ; then
        # " DB2 Install Location: $DB2V97PATH"
        # DB2 Version 
        echo ""
    else
        # " ERROR: Invalid DB2 Install Location specified
        echo "Return 15" >> ${TKLMDBMIGR_LOG}
        exit 15 
    fi
  else
    # " ERROR: Invalid DB2 Install Location specified
    echo "Return 15" >> ${TKLMDBMIGR_LOG}
    exit 15 
  fi

  if [ $DB2OLOC ] ; then
    if [[ -r $DB2OLOC ]] ; then
        echo ""
        # " DB2 Install Location: $DB2V9PATH"
        # DB2 Version 
    else
        # " ERROR: Invalid DB2 Install Location specified
        echo "Return 20" >> ${TKLMDBMIGR_LOG}
        exit 20 
    fi
  else
    # " ERROR: Invalid DB2 Install Location specified
    echo "Return 20" >> ${TKLMDBMIGR_LOG}
    exit 20 
  fi

test_DB2_INSTANCE

if [[ "${NEED_TO_MIGRATE_DB2_INST}" = "" || "${NEED_TO_MIGRATE_DB2_INST}" != "false" ]] ; then
  NEED_TO_MIGRATE_DB2_INST="true"
fi

get_DB2_Version
pre_migration
ckmigrate_db
migrate_instance
set_comm
set_svcename

echo " "
echo "Return=0"
echo "Return=0" >> ${TKLMDBMIGR_LOG}
exit 0
