#!/bin/sh
#set -x
#
# Licensed Materials-Property of IBM
# (C)Copyright IBM Corp 2008
# All rights reserved
# US Government Users Restricted Rights
# Use, duplication, or disclosure restricted
# by GSA ADP Schedule Contract with IBM Corp
# 
# The script is to be called from the IBM Tivoli Key Life Cycle Manager 
# Installer
#
# arg1 -  log  -   log file to be used by this script 
# arg2 -  instance  -  instance name (for example db2inst2) 
# arg3 -  dbname  -  database name
# arg4 -  dbinstance_owner_home - home of the instance owner  
# arg5 -  dbbackupdir  -  directory containing backup file
# arg6 -  dbtimestamp  -  backup time stamp
# arg7 -  dbstoptime -  backup time stamp - not used in FP 1 and later versions
#  Note:  The script 
#	        must be run on the same machine where DB2 server is installed.
#		does do not parameter validation
#
#------------------------------------------------------------------------------

LOG=$1
DB_INST=$2
DB_NAME=$3
INST_HOME=$4
DB_BACKUPDIR=$5

#The return code from the script
rc=0

if [ $# -ne 5 ]
then
echo Usage: dbmigbackup.sh logfile dbinstance dbname dbinstance_owner_home dbbackupdir
    exit 2
fi


##################################################################
echo "---------------------------------------------------------------------------------------------------" >> $LOG
echo Started executing dbmigbackup.sh >> "$LOG"
echo "---------------------------------------------------------------------------------------------------" >> $LOG

CURRENT_DIR=`dirname $0`
if [ "$CURRENT_DIR" = "." ]; then
   CURRENT_DIR=`pwd`
fi

chmod 777 $DB_BACKUPDIR

# set instance profile path to source profile of local instance

#source $DB_INST/db2profile
echo . $INST_HOME/sqllib/db2profile >> "$LOG" 
. $INST_HOME/sqllib/db2profile
rc=$?
   
echo db2 stop database manager force >> "$LOG"
db2 -l "$LOG" "stop database manager force"

echo db2 start database manager >> "$LOG"
db2 -l "$LOG" "start database manager"

echo "db2 list applications" >> "$LOG"
db2 "list applications" >> "$LOG"

echo "db2 backup DB $DB_NAME to $DB_BACKUPDIR" >> "$LOG"
db2 "backup DB $DB_NAME to $DB_BACKUPDIR" >> "$LOG"
rc=$?
if [ $rc -gt 0 ] && [ $rc -lt 4 ]; then
   rc=0
fi

echo "---------------------------------------------------------------------------------------------------" >> $LOG
if [ $rc -eq 0 ]; then
echo "Return=$rc"
echo "Return=$rc" >> $LOG
else
echo "Return $rc" >> $LOG
fi
echo "---------------------------------------------------------------------------------------------------" >> $LOG
exit $rc

